/*** release_rom.cmd - fpalette boot-loader (rom) link command file ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/05/17 mimu
              clean-up
              change memory assign
 * 2011/02/xx mimu
              First Version
 */
/*
 * <<< Initialized Sections Created by the Compiler >>>
 * +---------+---------------------------------------------------------------+-------------------------
 * | Name    | Contents                                                      | Restrictions
 * +---------+---------------------------------------------------------------+-------------------------
 * | .cinit  | Tables for explicitly initialized global and static variables | Program
 * +---------+---------------------------------------------------------------+-------------------------
 * | .const  | Global and static const variables that are explicitly         | Low 64K
 * |         | initialized and contain string literals                       |
 * +---------+---------------------------------------------------------------+-------------------------
 * | .econst | Far constant variables                                        | Anywhere in data
 * +---------+---------------------------------------------------------------+-------------------------
 * | .pinit  | Table of constructors to be called at startup                 | Program
 * +---------+---------------------------------------------------------------+-------------------------
 * | .switch | Jump tables for large switch statements                       | Program with -mt option
 * |         |                                                               | Data without -mt option
 * +---------+---------------------------------------------------------------+-------------------------
 * | .text   |Executable code and constants                                  | Program
 * +---------+---------------------------------------------------------------+-------------------------
 *
 * <<< Uninitialized Sections Created by the Compiler >>>
 * +----------+------------------------------------+-----------------------
 * | Name     | Contents                           | Restrictions
 * +----------+------------------------------------+-----------------------
 * | .bss     | Global and static variables        | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .ebss    | Far global/static variables        | Anywhere in data
 * +----------+------------------------------------+-----------------------
 * | .stack   | Stack                              | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .sysmem  | Memory for malloc functions (heap) | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .esysmem | Memory for far_malloc functions    | Anywhere in data
 * +----------+------------------------------------+-----------------------
 *
 * <<< Additional Sections by program >>>
 * +-----------+------------------------------------+-----------------------
 * | Name      | Contents                           | Restrictions
 * +-----------+------------------------------------+-----------------------
 * | codestart |
 * +-----------+------------------------------------+-----------------------
 * | ramfuncs  |
 * +-----------+------------------------------------+-----------------------
 * | .reset    |
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 *
 * <<< Section Linking in Non-DSP/BIOS Projects (Large memory model) >>>
 * +--------------+----------------+
 * | Section Name | Where to Link  |
 * +--------------+----------------+
 * | .cinit       | Flash          |
 * | .cio         | RAM            |
 * | .const       | Flash          |
 * | .econst      | Flash          |
 * | .pinit       | Flash          |
 * | .switch      | Flash          |
 * | .text        | Flash          |
 * | .bss         | RAM            |
 * | .ebss        | RAM            |
 * | .stack       | Lower 64Kw RAM |
 * | .sysmem      | RAM            |
 * | .esysmem     | RAM            |
 * | .reset       | RAM(1)         |
 * +--------------+----------------+
 * (1) The .reset section contains nothing more than a 32-bit interrupt vector
 * that points to the C-compiler boot function in the runtime support library
 * (the _c_int00 routine). It generally is not used. Instead, the user typically
 * creates their own branch instruction to point to the starting point of the
 * code (see Sections 6 and 7). When not in use, the .reset section should be
 * omitted from the code build by using a DSECT modifier in the linker command
 * file.
 */

MEMORY
{
	/*
	 * [PAGE n : ] name [(attr)] : origin = constant, length = constant[, fill = constant]
	 * PAGE 0 : text
	 * PAGE 1 : data
	 * PAGE 2 : used by boot-rom
	 * PAGE 3 : user application save area (flash)
	 */
	/*
	 * On chip memory
	 *
	 *	M0_SRAM		(RWXI)	: origin = 0x000000, length = 0x0400	?*  1K x 16 SARAM	00:0000 - 00:03ff *?
	 *	M1_SRAM		(RWXI)	: origin = 0x000400, length = 0x0400	?*  1K x 16 SARAM	00:0400 - 00:07ff *?
	 *
	 *	L0_SRAM		(RWXI)	: origin = 0x008000, length = 0x0800	?*  2K x 16 SRAM	00:8000 - 00:87ff *?
	 *	L1_SRAM		(RWXI)	: origin = 0x008800, length = 0x0400	?*  1K x 16 DP-SRAM	00:8800 - 00:8bff *?
	 *	L2_SRAM		(RWXI)	: origin = 0x008c00, length = 0x0400	?*  1K x 16 DP-SRAM	00:8c00 - 00:8fff *?
	 *	L3_SRAM		(RWXI)	: origin = 0x009000, length = 0x1000	?*  4K x 16 DP-SRAM	00:9000 - 00:9fff *?
	 *
	 *	FLASH_H		(RX)	: origin = 0x3e8000, length = 0x2000	?*  8K x 16 SectorH 3e:8000 - 3e:9fff *?
	 *	FLASH_G		(RX)	: origin = 0x3ea000, length = 0x2000	?*  8K x 16 SectorG 3e:8000 - 3e:9fff *?
	 *	FLASH_F		(RX)	: origin = 0x3ec000, length = 0x2000	?*  8K x 16 SectorF 3e:8000 - 3e:9fff *?
	 *	FLASH_E		(RX)	: origin = 0x3ee000, length = 0x2000	?*  8K x 16 SectorE 3e:8000 - 3e:9fff *?
	 *	FLASH_D		(RX)	: origin = 0x3f0000, length = 0x2000	?*  8K x 16 SectorE 3e:8000 - 3e:9fff *?
	 *	FLASH_C		(RX)	: origin = 0x3f2000, length = 0x2000	?*  8K x 16 SectorC 3e:8000 - 3e:9fff *?
	 *	FLASH_B		(RX)	: origin = 0x3f4000, length = 0x2000	?*  8K x 16 SectorB 3e:8000 - 3e:9fff *?
	 *	FLASH_A		(RX)	: origin = 0x3f6000, length = 0x2000	?*  8K x 16 SectorA 3e:8000 - 3e:9fff *?
	 *						  origin = 0x3f7f80, length = 0x0076	?* Program to 0x0000 When using the Code Security Module *?
	 *						  origin = 0x3f7ff6, length = 0x0002	?* Boot-to-Flash Entry Point (program branch instruction here) *?
	 *						  origin = 0x3f7ff8, length = 0x0008	?* Security Password (128bits) (Do not program to all zeros)
	 */

/*	PAGE 0 : ENTRY		(RWXI)	: origin = 0x0000, length = 0x0002 */	/* for debug */
	PAGE 0 : _bootrom_			: origin = 0x0002, length = 0x004e
	PAGE 0 : SRAM00		(RWXI)	: origin = 0x0050, length = 0x03b0
	PAGE 0 : SRAM01		(RWXI)	: origin = 0x0400, length = 0x0400
	PAGE 0 : SRAM		(RWXI)	: origin = 0x8000, length = 0x2000


	PAGE 0 : PROG_ENTRY (RX)	: origin = 0x3e8000, length = 0x0002	/* User Program Entry */
	PAGE 0 : PROG		(RX)	: origin = 0x3e8002, length = 0x7ffe	/* Flash memory for User Program */

	PAGE 0 : LOADER		(RX)	: origin = 0x3f4000, length = 0x3f80	/* Flash memory for Boot loader */

	PAGE 0 : CSM_RSVD	(R)		: origin = 0x3f7f80, length = 0x0076	/* Part of FLASH Sector A.  Reserved when CSM is in use. */
	PAGE 0 : ENTRY		(RI)	: origin = 0x3f7ff6, length = 0x0002	/* Part of FLASH Sector A.  Used for "Jump to flash" bootloader mode. */
	PAGE 0 : PASSWORDS	(R)		: origin = 0x3f7ff8, length = 0x0008	/* Part of FLASH Sector A.  CSM password locations. (128bits) */

}


SECTIONS
{

	.reset		:                                   PAGE = 0, TYPE = DSECT


	.stack		: > SRAM01,                         PAGE = 0

	.bss		: > SRAM,                           PAGE = 0
	.ebss		: > SRAM,                           PAGE = 0
	.cio		: > SRAM,                           PAGE = 0
	.sysmem		: > SRAM,                           PAGE = 0
	.esysmem	: > SRAM,                           PAGE = 0

	codestart	: > ENTRY,                          PAGE = 0

	.cinit		: > LOADER,                         PAGE = 0
	.text		: > LOADER,                         PAGE = 0
	.econst		: > LOADER,                         PAGE = 0
	.pinit		: > LOADER,                         PAGE = 0
	.switch		: > LOADER,                         PAGE = 0

	ramfuncs	: LOAD = LOADER,                    PAGE = 0
	              RUN = SRAM00,
	              LOAD_START(_ramfuncs_loadstart),
	              LOAD_SIZE( _ramfuncs_loadsize),
	              RUN_START( _ramfuncs_runstart)

	csm_rsvd	: > CSM_RSVD,                       PAGE = 0
	passwords	: > PASSWORDS,                      PAGE = 0

}

/*** end of file : release_rom.cmd ***/
