/*** dspCtrl.c - TMS320F28035 DSP control (clock, PLL, Power, watchdog) ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/19 mimu
              First Version
 */

/*** NOTE ***
 * XCLK         0x7010 1 XCLKOUT/XCLKIN Control
 * PLLSTS       0x7011 1 PLL Status
 * CLKCTL       0x7012 1 Clock Control
 * PLLLOCKPRD   0x7013 1 PLL Lock Period
 * INTOSC1TRIM  0x7014 1 Internal Oscillator 1 Trim
 * INTOSC2TRIM  0x7016 1 Internal Oscillator 2 Trim
 *
 * LOSPCP       0x701B 1 Low-Speed Peripheral Clock Pre-Scaler
 * PCLKCR0      0x701C 1 Peripheral Clock Control 0
 * PCLKCR1      0x701D 1 Peripheral Clock Control 1
 * LPMCR0       0x701E 1 Low Power Mode Control 0
 * PCLKCR3      0x7020 1 Peripheral Clock Control 3
 * PLLCR        0x7021 1 PLL Control
 *
 * SCSR         0x7022 1 System Control & Status
 * WDCNTR       0x7023 1 Watchdog Counter
 *
 * WDKEY        0x7025 1 Watchdog Reset Key
 *
 * WDCR         0x7029 1 Watchdog Control
 *
 * BORCFG       0x0985 1 BOR Configuration
 */

/*** INCLUDE ***/
#include "config.h"

#include "tms320f28035.h"
#include "dspCtrl.h"

/*** DEFINE ***/
/*** XCLK ***/
#define	XCLK		(volatile unsigned short *)0x7010	/* XCLKOUT/XCLKIN Control					*/
#define	XCLK_XCLKINSEL	(1<<6)	/* XCLKIN Source Select */
#define	XCLK_XCLKOUTDIV_4	(0)	/* XCLKOUT = SYSCLKOUT/4 */
#define	XCLK_XCLKOUTDIV_2	(1)	/* XCLKOUT = SYSCLKOUT/2 */
#define	XCLK_XCLKOUTDIV_1	(2)	/* XCLKOUT = SYSCLKOUT/1 */
#define	XCLK_XCLKOUTDIV_Off	(3)	/* XCLKOUT = Off         */

/*** PLLSTS ***/
#define	PLLSTS		(volatile unsigned short *)0x7011	/* PLL Status								*/
#define	PLLSTS_NORMRDYE	(1<<15)	/* NORMRDY Enable */

#define	PLLSTS_DIVSEL_MASK	(3<<7)
#define	PLLSTS_DIVSEL_4		(0<<7)	/* Select Divide By 4 for CLKIN */
#define	PLLSTS_DIVSEL_2		(2<<7)	/* Select Divide By 2 for CLKIN */
#define	PLLSTS_DIVSEL_1		(3<<7)	/* Select Divide By 1 for CLKIN. (This mode can be used only when PLL is off or bypassed.) */
#define	PLLSTS_DIVSEL(_v)	(((_v)&0x03) <<7)
#define	PLLSTS_MCLKOFF		(1<<6)	/* Missing clock-detect off */
#define	PLLSTS_OSCOFF		(1<<5)	/* Oscillator Clock Off */
#define	PLLSTS_MCLKCLR		(1<<4)	/* Missing Clock Clear */
#define	PLLSTS_MCLKSTS		(1<<3)	/* Missing Clock Status */
#define	PLLSTS_PLLOFF		(1<<2)	/* PLL Off */
#define	PLLSTS_PLLLOCKS		(1<<0)	/* PLL Lock Status */

/*** CLKCTL ***/
#define	CLKCTL		(volatile unsigned short *)0x7012	/* Clock Control							*/
#define	CLKCTL_NMIRESETSEL	(1<<15)	/* NMI Reset Select */
#define	CLKCTL_XTALOSCOFF	(1<<14)	/* Crystal Oscillator Off */
#define	CLKCTL_XCLKINOFF	(1<<13)	/* XCLKIN Off */
#define	CLKCTL_WDHALTI	(1<<12)	/* Watchdog HALT Mode Ignore */
#define	CLKCTL_INTOSC2HALTI	(1<<11)	/* Internal Oscillator 2 HALT Mode Ignore */
#define	CLKCTL_INTOSC2OFF	(1<<10)	/* Internal Oscillator 2 Off */
#define	CLKCTL_INTOSC1HALTI	(1<<9)	/* Internal Oscillator 1 HALT Mode Ignore */
#define	CLKCTL_INTOSC1OFF	(1<<8)	/* Internal Oscillator 1 Off */

#define	CLKCTL_TMR2CLKPRESCALE_1	(0<<5)	/* CPU Timer 2 Clock Pre-Scale Value */
#define	CLKCTL_TMR2CLKPRESCALE_2	(1<<5)
#define	CLKCTL_TMR2CLKPRESCALE_4	(2<<5)
#define	CLKCTL_TMR2CLKPRESCALE_8	(3<<5)
#define	CLKCTL_TMR2CLKPRESCALE_16	(4<<5)
												/* CPU Timer 2 Clock Source Select */
#define	CLKCTL_TMR2CLKSRCSEL_SYSCLKOUT	(0<<3)	/* SYSCLKOUT Selected (default on reset, pre-scaler is bypassed) */
#define	CLKCTL_TMR2CLKSRCSEL_EXT_OSC	(1<<3)	/* External Oscillator Selected (at XOR output) */
#define	CLKCTL_TMR2CLKSRCSEL_OSC1		(2<<3)	/* Internal Oscillator 1 Selected */
#define	CLKCTL_TMR2CLKSRCSEL_OSC2		(3<<3)	/* Internal Oscillator 2 Selected. */
#define	CLKCTL_WDCLKSRCSEL		(1<<2)	/* Watchdog Clock Source Select Bit */
#define	CLKCTL_OSCCLKSRC2SEL	(1<<1)	/* Oscillator 2 Clock Source Select Bit */
#define	CLKCTL_OSCCLKSRCSEL		(1<<0)	/* Oscillator Clock Source Select Bit */



#define	PLLLOCKPRD	(volatile unsigned short *)0x7013	/* PLL Lock Period							*/

/*** INTOSCxTRIM ***/
#define	INTOSC1TRIM	(volatile unsigned short *)0x7014	/* Internal Oscillator 1 Trim				*/
#define	INTOSC2TRIM	(volatile unsigned short *)0x7016	/* Internal Oscillator 2 Trim				*/

#define	INTOSCxTRIM_FINETRIM(_v)	(((_v)&0x3f)<<9)
#define	INTOSCxCOARSETRIM(_v)		((_v) & 0xff)


/*
 * for clock
 */
/*** LOSPCP ***/
#define	LOSPCP		(volatile unsigned short *)0x701B	/* Low-Speed Peripheral Clock Pre-Scaler	*/
#define	LOSPCP_LSPCLK_1		0	/* Low speed clock = SYSCLKOUT/1  */
#define	LOSPCP_LSPCLK_2		1	/* Low speed clock = SYSCLKOUT/2  */
#define	LOSPCP_LSPCLK_4		2	/* Low speed clock = SYSCLKOUT/4  (default) */
#define	LOSPCP_LSPCLK_6		3	/* Low speed clock = SYSCLKOUT/6  */
#define	LOSPCP_LSPCLK_8		4	/* Low speed clock = SYSCLKOUT/8  */
#define	LOSPCP_LSPCLK_10	5	/* Low speed clock = SYSCLKOUT/10 */
#define	LOSPCP_LSPCLK_12	6	/* Low speed clock = SYSCLKOUT/12 */
#define	LOSPCP_LSPCLK_14	7	/* Low speed clock = SYSCLKOUT/14 */

/** PCLKCR0 ***/
#define	PCLKCR0		(volatile unsigned short *)0x701C	/* Peripheral Clock Control 0				*/
#define	PCLKCR0_ECANAENCLK	(1<<14)	/* ECAN-A clock enable			*/
#define	PCLKCR0_SCIAENCLK	(1<<10)	/* SCI-A clock enable			*/
#define	PCLKCR0_SPIBENCLK	(1<<9)	/* SPIBENCLK SPI-B clock enable	*/
#define	PCLKCR0_SPIAENCLK	(1<<8)	/* SPI-A clock enable			*/
#define	PCLKCR0_I2CAENCLK	(1<<4)	/* I2C clock enable				*/
#define	PCLKCR0_ADCENCLK	(1<<3)	/* ADC clock enable				*/
#define	PCLKCR0_TBCLKSYNC	(1<<2)	/* ePWM Module Time Base Clock (TBCLK) Sync
									 * : Allows the user to globally synchronize all enabled
									 * ePWM modules to the time base clock (TBCLK):
									 */
#define	PCLKCR0_LINAENCLK	(1<<1)	/* LIN-A clock enable.			*/
#define	PCLKCR0_HRPWMENCLK	(1<<0)	/*  HRPWM clock enable			*/

/** PCLKCR1 ***/
#define	PCLKCR1		(volatile unsigned short *)0x701D	/* Peripheral Clock Control 1				*/
#define	PCLKCR1_EQEP1ENCLK	(1<<14)	/* eQEP1 clock enable	*/
#define	PCLKCR1_ECAP1ENCLK	(1<<8)	/* eCAP1 clock enable	*/
#define	PCLKCR1_EPWM7ENCLK	(1<<6)	/* ePWM7 clock enable	*/
#define	PCLKCR1_EPWM6ENCLK	(1<<5)	/* ePWM6 clock enable	*/
#define	PCLKCR1_EPWM5ENCLK	(1<<4)	/* ePWM5 clock enable	*/
#define	PCLKCR1_EPWM4ENCLK	(1<<3)	/* ePWM4 clock enable	*/
#define	PCLKCR1_EPWM3ENCLK	(1<<2)	/* ePWM3 clock enable	*/
#define	PCLKCR1_EPWM2ENCLK	(1<<1)	/* ePWM2 clock enable	*/
#define	PCLKCR1_EPWM1ENCLK	(1<<0)	/* ePWM1 clock enable	*/

/** LPMCR0 ***/
#define	LPMCR0		(volatile unsigned short *)0x701E	/* Low Power Mode Control 0					*/
#define	LPMCR0_WDINTE	(1<<15)	/* Watchdog interrupt enable */
#define	LPMCR0_QUALSTDBY(_v)	( ((_v)&0x3f) << 2 )
#define	LPMCR0_LPM_IDLE		0	/* Set the low power mode to IDLE (default) */
#define	LPMCR0_LPM_STANDBY	1	/* Set the low power mode to STANDBY */
#define	LPMCR0_LPM_HALT2	2	/* Set the low power mode to HALT */
#define	LPMCR0_LPM_HALT3	3	/* Set the low power mode to HALT */

/** PCLKCR3 ***/
#define	PCLKCR3		(volatile unsigned short *)0x7020	/* Peripheral Clock Control 3				*/
#define	PCLKCR3_CLA1ENCLK		(1<<14)	/* CLA module clock enable	*/
#define	PCLKCR3_GPIOINENCLK		(1<<13)	/* GPIO Input Clock Enable	*/
#define	PCLKCR3_CPUTIMER2ENCLK	(1<<10)	/* CPU Timer 2 Clock Enable	*/
#define	PCLKCR3_CPUTIMER1ENCLK	(1<<9)	/* CPU Timer 1 Clock Enable	*/
#define	PCLKCR3_CPUTIMER0ENCLK	(1<<8)	/* CPU Timer 0 Clock Enable	*/
#define	PCLKCR3_COMP3ENCLK		(1<<2)	/* Comparator3 clock enable	*/
#define	PCLKCR3_COMP2ENCLK		(1<<1)	/* Comparator2 clock enable	*/
#define	PCLKCR3_COMP1ENCLK		(1<<0)	/* Comparator1 clock enable	*/

/** PLLCR ***/
#define	PLLCR		(volatile unsigned short *)0x7021	/* PLL Control								*/
#define	PLLCR_BYPASS	0
#define	PLLCR_1			1
#define	PLLCR_2			2
#define	PLLCR_3			3
#define	PLLCR_4			4
#define	PLLCR_5			5
#define	PLLCR_6			6
#define	PLLCR_7			7
#define	PLLCR_8			8
#define	PLLCR_9			9
#define	PLLCR_10		10
#define	PLLCR_11		11
#define	PLLCR_12		12

/*** SCSR ***/
#define	SCSR		(volatile unsigned short *)0x7022	/* System Control & Status					*/

#define	SCSR_WDINTS		(1<<2)	/* Watchdog interrupt status */
#define	SCSR_WDENINT	(1<<1)	/* Watchdog interrupt enable */
#define	SCSR_WDOVERRIDE	(1<<0)	/* Watchdog override */

/*** WDCNTR ***/
#define	WDCNTR		(volatile unsigned short *)0x7023	/* Watchdog Counter							*/

/*** WDKEY ***/
#define	WDKEY		(volatile unsigned short *)0x7025	/* Watchdog Reset Key						*/

/*** WDCR ***/
#define	WDCR		(volatile unsigned short *)0x7029	/* Watchdog Control							*/
#define	WDCR_WDFLAG		(1<<7)	/* Watchdog reset status flag */
#define	WDCR_WDDIS		(1<<6)	/* Watchdog disable */
#define	WDCR_WDCHK_101	(5<<3)
#define	WDCR_WDCHK_MSK	(7<<3)
			/* Watchdog pre-sclase */
#define	WDCR_WDPS_1		1	/* WDCLK = OSCCLK/512/1 */
#define	WDCR_WDPS_2		2	/* WDCLK = OSCCLK/512/2 */
#define	WDCR_WDPS_4		3	/* WDCLK = OSCCLK/512/4 */
#define	WDCR_WDPS_8		4	/* WDCLK = OSCCLK/512/8 */
#define	WDCR_WDPS_16	5	/* WDCLK = OSCCLK/512/16 */
#define	WDCR_WDPS_32	6	/* WDCLK = OSCCLK/512/32 */
#define	WDCR_WDPS_64	7	/* WDCLK = OSCCLK/512/64 */

/*** NMICFG ***/
#define	NMICFG		(volatile unsigned short *)0x7060	/* NMI Configuration Register				*/
#define	NMICFG_CLOCKFAIL	(1<<1)	/* CLOCKFAIL-interrupt Enable */

/*** NMIFLG/NMIFLGCLR/NMIFLGFRC ***/
#define	NMIFLG		(volatile unsigned short *)0x7061	/* NMI Flag Register						*/
#define	NMIFLGCLR	(volatile unsigned short *)0x7062	/* NMI Flag Clear Register					*/
#define	NMIFLGFRC	(volatile unsigned short *)0x7063	/* NMI Flag Force Register					*/

#define	NMIFLG_CLOCKFAIL	(1<<1)	/* CLOCKFAIL Interrupt Flag */
#define	NMIFLG_NMIINT		(1<<0)	/* NMI Interrupt Flag */

#define	NMIWDCNT	(volatile unsigned short *)0x7064	/* NMI Watchdog Counter Register			*/
/*** NMIWDPRD ***/
#define	NMIWDPRD	(volatile unsigned short *)0x7065	/* NMI Watchdog Period Register				*/
#define	BORCFG		(volatile unsigned short *)0x0985	/* BOR Configuration						*/

/*** FUNCTION ***/
//static void ServiceDog(void);
static void DisableDog(void);
static void InitPll(unsigned short val, unsigned short divsel);
static void InitPeripheralClocks(void);
//static unsigned short CsmUnlock();
static void IntOsc1Sel (void);
//static void IntOsc2Sel (void);
//static void XtalOscSel (void);
//static void ExtOscSel (void);


/*** WORK ***/


/*---------------------------------------------------------------------------
 * InitSysCtrl:
 *---------------------------------------------------------------------------
 * This function initializes the System Control registers to a known state.
 * - Disables the watchdog
 * - Set the PLLCR for proper SYSCLKOUT frequency
 * - Set the pre-scaler for the high and low frequency peripheral clocks
 * - Enable the clocks to the peripherals
 */

void InitSysCtrl(void)
{

	/* Disable the watchdog */
	DisableDog();

	/* *IMPORTANT*
	 * The Device_cal function, which copies the ADC & oscillator calibration values
	 * from TI reserved OTP into the appropriate trim registers, occurs automatically
	 * in the Boot ROM. If the boot ROM code is bypassed during the debug process, the
	 * following function MUST be called for the ADC and oscillators to function according
	 * to specification. The clocks to the ADC MUST be enabled before calling this
	 * function.
	 * See the device data manual and/or the ADC Reference
	 * Manual for more information.
	 */

	EALLOW;
//		SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1; // Enable ADC peripheral clock
		*PCLKCR0 |= PCLKCR0_ADCENCLK;	/* Enable ADC peripheral clock */
		(*Device_cal)();
//		SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 0; // Return ADC clock to original state
		*PCLKCR0 &= ~PCLKCR0_ADCENCLK;	/* Return ADC clock to original state */
	EDIS;

	/* Select Internal Oscillator 1 as Clock Source (default), and turn off all unused clocks to
	 * conserve power.
	 */
	IntOsc1Sel();

	/* Initialize the PLL control: PLLCR and CLKINDIV
	 * DSP28_PLLCR and DSP28_CLKINDIV are defined in DSP2803x_Examples.h
	 */
	InitPll(DSP28_PLLCR,DSP28_DIVSEL);

	/* Initialize the peripheral clocks */
	InitPeripheralClocks();

	flashInit();
}


/*---------------------------------------------------------------------------
 * Example: ServiceDog:
 *---------------------------------------------------------------------------
 * This function resets the watchdog timer.
 * Enable this function for using ServiceDog in the application
 */
#if 0
static void ServiceDog(void)
{
	EALLOW;
//		SysCtrlRegs.WDKEY = 0x0055;
//		SysCtrlRegs.WDKEY = 0x00AA;
		*WDKEY = 0x0055;
		*WDKEY = 0x00aa;
	EDIS;
}
#endif

/*---------------------------------------------------------------------------
 * Example: DisableDog:
 *---------------------------------------------------------------------------
 * This function disables the watchdog timer.
 */
static void DisableDog(void)
{
	EALLOW;
//		SysCtrlRegs.WDCR= 0x0068;
		*WDCR = WDCR_WDDIS | WDCR_WDCHK_101;
	EDIS;
}

/*---------------------------------------------------------------------------
 * Example: InitPll:
 *---------------------------------------------------------------------------
 * This function initializes the PLLCR register.
 */
static void InitPll(unsigned short val, unsigned short divsel)
{
//	volatile unsigned short iVol;

	/*
	 * Make sure the PLL is not running in limp mode
	 */
//	if (SysCtrlRegs.PLLSTS.bit.MCLKSTS != 0)
	if( *PLLSTS & PLLSTS_MCLKSTS )
	{
	EALLOW;
		/*
		 * OSCCLKSRC1 failure detected. PLL running in limp mode.
		 * Re-enable missing clock logic.
		 */
//		SysCtrlRegs.PLLSTS.bit.MCLKCLR = 1;
		*PLLSTS |= PLLSTS_MCLKCLR;
		sysHalt( "PLL limp mode detect!!" );
	EDIS;

	/*
	 * Replace this line with a call to an appropriate
	 * SystemShutdown(); function.
	 */
	asm("        ESTOP0");     /* Uncomment for debugging purposes */
	}

	/*
	 * DIVSEL MUST be 0 before PLLCR can be changed from
	 * 0x0000. It is set to 0 by an external reset XRSn
	 * This puts us in 1/4
	 */
//	if (SysCtrlRegs.PLLSTS.bit.DIVSEL != 0)
	if( *PLLSTS & PLLSTS_DIVSEL_MASK )
	{
		EALLOW;
//			SysCtrlRegs.PLLSTS.bit.DIVSEL = 0;
			*PLLSTS &= ~PLLSTS_DIVSEL_MASK;
		EDIS;
	}

	/*
	 * Change the PLLCR
	 */
//	if (SysCtrlRegs.PLLCR.bit.DIV != val)
	if( *PLLCR != val )
	{
		EALLOW;
			/*
			 * Before setting PLLCR turn off missing clock detect logic
			 */
//			SysCtrlRegs.PLLSTS.bit.MCLKOFF = 1;
			*PLLSTS |= PLLSTS_MCLKOFF;
//			SysCtrlRegs.PLLCR.bit.DIV = val;
			*PLLCR = val;
		EDIS;

		/*
		 * Optional: Wait for PLL to lock.
		 * During this time the CPU will switch to OSCCLK/2 until
		 * the PLL is stable.  Once the PLL is stable the CPU will
		 * switch to the new PLL value.
		 *
		 * This time-to-lock is monitored by a PLL lock counter.
		 *
		 * Code is not required to sit and wait for the PLL to lock.
		 * However, if the code does anything that is timing critical,
		 * and requires the correct clock be locked, then it is best to
		 * wait until this switching has completed.
		 *
		 * Wait for the PLL lock bit to be set.
		 *
		 * The watchdog should be disabled before this loop, or fed within
		 * the loop via ServiceDog().
		 *
		 * Uncomment to disable the watchdog
		 */
		DisableDog();

//		while(SysCtrlRegs.PLLSTS.bit.PLLLOCKS != 1)
		while( (*PLLSTS & PLLSTS_PLLLOCKS) == 0 )
		{
			/*
			 * Uncomment to service the watchdog
			 */
			// ServiceDog();
		}

		EALLOW;
//			SysCtrlRegs.PLLSTS.bit.MCLKOFF = 0;
			*PLLSTS &= PLLSTS_MCLKOFF;
		EDIS;
	}

#if 1
	EALLOW;
	{
		unsigned short pllstsVal;

		pllstsVal = *PLLSTS;
		pllstsVal &= ~PLLSTS_DIVSEL_MASK;

		switch( divsel ) {
		case 1:
		case 2:
			*PLLSTS = pllstsVal | PLLSTS_DIVSEL( divsel );
			break;
		case 3:
			*PLLSTS = pllstsVal | PLLSTS_DIVSEL( 2 );
			DELAY_US(50L);
			*PLLSTS = pllstsVal | PLLSTS_DIVSEL( 3 );
			break;
		}
	}
	EDIS;
#else
	/*
	 * If switching to 1/2
	 */
	if((divsel == 1)||(divsel == 2))
	{
		EALLOW;
			SysCtrlRegs.PLLSTS.bit.DIVSEL = divsel;
		EDIS;
	}

	/*
	 * If switching to 1/1
	 * * First go to 1/2 and let the power settle
	 *   The time required will depend on the system, this is only an example
	 * * Then switch to 1/1
	 */
	if(divsel == 3)
	{
		EALLOW;
			SysCtrlRegs.PLLSTS.bit.DIVSEL = 2;
			DELAY_US(50L);
			SysCtrlRegs.PLLSTS.bit.DIVSEL = 3;
		EDIS;
	}
#endif
}

/*--------------------------------------------------------------------------
 * Example: InitPeripheralClocks:
 *---------------------------------------------------------------------------
 * This function initializes the clocks to the peripheral modules.
 * First the high and low clock prescalers are set
 * Second the clocks are enabled to each peripheral.
 * To reduce power, leave clocks to unused peripherals disabled
 *
 * Note: If a peripherals clock is not enabled then you cannot
 * read or write to the registers for that peripheral
 */
static void InitPeripheralClocks(void)
{
	EALLOW;

		/*
		 * LOSPCP prescale register settings, normally it will be set to default values
		 */

#if 0 // delete by mimu
		GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 3;  /* GPIO18 = XCLKOUT */
#endif
//		SysCtrlRegs.LOSPCP.all = 0x0002;
		*LOSPCP = LOSPCP_LSPCLK_4;

		/*
		 * XCLKOUT to SYSCLKOUT ratio.  By default XCLKOUT = 1/4 SYSCLKOUT
		 */
//		SysCtrlRegs.XCLK.bit.XCLKOUTDIV=2;
		*XCLK = XCLK_XCLKOUTDIV_1;

		/*
		 * Peripheral clock enables set for the selected peripherals.
		 * If you are not using a peripheral leave the clock off
		 * to save on power.
		 *
		 * Note: not all peripherals are available on all 2803x derivates.
		 * Refer to the datasheet for your particular device.
		 *
		 * This function is not written to be an example of efficient code.
		 */

//		SysCtrlRegs.PCLKCR0.bit.ADCENCLK   = 1;	/* ADC								*/
//		SysCtrlRegs.PCLKCR0.bit.ECANAENCLK = 1;	/* eCAN-A							*/
//		SysCtrlRegs.PCLKCR0.bit.HRPWMENCLK = 1;	/* HRPWM							*/
//		SysCtrlRegs.PCLKCR0.bit.I2CAENCLK  = 1;	/* I2C								*/
//		SysCtrlRegs.PCLKCR0.bit.LINAENCLK  = 1;	/* LIN-A							*/
//		SysCtrlRegs.PCLKCR0.bit.SCIAENCLK  = 1;	/* SCI-A							*/
//		SysCtrlRegs.PCLKCR0.bit.SPIAENCLK  = 1;	/* SPI-A							*/
//		SysCtrlRegs.PCLKCR0.bit.SPIBENCLK  = 1;	/* SPI-B							*/
//		SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC  = 1;	/* Enable TBCLK within the ePWM		*/
		*PCLKCR0 = 0
		         | PCLKCR0_ECANAENCLK		/* ECAN-A clock enable			*/
		         | PCLKCR0_SCIAENCLK		/* SCI-A clock enable			*/
		         | PCLKCR0_SPIBENCLK		/* SPIBENCLK SPI-B clock enable	*/
		         | PCLKCR0_SPIAENCLK		/* SPI-A clock enable			*/
		         | PCLKCR0_I2CAENCLK		/* I2C clock enable				*/
		         | PCLKCR0_ADCENCLK			/* ADC clock enable				*/
		         | PCLKCR0_TBCLKSYNC		/* ePWM Module Time Base Clock (TBCLK) Sync */
		         | PCLKCR0_LINAENCLK		/* LIN-A clock enable.			*/
		         | PCLKCR0_HRPWMENCLK		/*  HRPWM clock enable			*/
		         ;

//		SysCtrlRegs.PCLKCR1.bit.ECAP1ENCLK = 1;	/* eCAP1							*/
//		SysCtrlRegs.PCLKCR1.bit.EQEP1ENCLK = 1;	/* eQEP1							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM1ENCLK = 1;	/* ePWM1							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM2ENCLK = 1;	/* ePWM2							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM3ENCLK = 1;	/* ePWM3							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM4ENCLK = 1;	/* ePWM4							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM5ENCLK = 1;	/* ePWM5							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM6ENCLK = 1;	/* ePWM6							*/
//		SysCtrlRegs.PCLKCR1.bit.EPWM7ENCLK = 1;	/* ePWM7							*/
		*PCLKCR1 = 0
		         | PCLKCR1_EQEP1ENCLK		/* eQEP1 clock enable	*/
		         | PCLKCR1_ECAP1ENCLK		/* eCAP1 clock enable	*/
		         | PCLKCR1_EPWM7ENCLK		/* ePWM7 clock enable	*/
		         | PCLKCR1_EPWM6ENCLK		/* ePWM6 clock enable	*/
		         | PCLKCR1_EPWM5ENCLK		/* ePWM5 clock enable	*/
		         | PCLKCR1_EPWM4ENCLK		/* ePWM4 clock enable	*/
		         | PCLKCR1_EPWM3ENCLK		/* ePWM3 clock enable	*/
		         | PCLKCR1_EPWM2ENCLK		/* ePWM2 clock enable	*/
		         | PCLKCR1_EPWM1ENCLK		/* ePWM1 clock enable	*/
		         ;

//		SysCtrlRegs.PCLKCR3.bit.COMP1ENCLK = 1;	/* COMP1							*/
//		SysCtrlRegs.PCLKCR3.bit.COMP2ENCLK = 1;	/* COMP2							*/
//		SysCtrlRegs.PCLKCR3.bit.COMP3ENCLK = 1;	/* COMP3							*/
//		SysCtrlRegs.PCLKCR3.bit.CLA1ENCLK  = 1;	/* CLA1								*/
		*PCLKCR3 = 0
		         | PCLKCR3_CLA1ENCLK			/* CLA module clock enable	*/
		         | PCLKCR3_GPIOINENCLK			/* GPIO Input Clock Enable	*/
		         | PCLKCR3_CPUTIMER2ENCLK		/* CPU Timer 2 Clock Enable	*/
		         | PCLKCR3_CPUTIMER1ENCLK		/* CPU Timer 1 Clock Enable	*/
		         | PCLKCR3_CPUTIMER0ENCLK		/* CPU Timer 0 Clock Enable	*/
		         | PCLKCR3_COMP3ENCLK			/* Comparator3 clock enable	*/
		         | PCLKCR3_COMP2ENCLK			/* Comparator2 clock enable	*/
		         | PCLKCR3_COMP1ENCLK			/* Comparator1 clock enable	*/
		         ;

	EDIS;
}

/*---------------------------------------------------------------------------
 * Example: CsmUnlock:
 *---------------------------------------------------------------------------
 * This function unlocks the CSM. User must replace 0xFFFF's with current
 * password for the DSP. Returns 1 if unlock is successful.
 */
#if 0
#define STATUS_FAIL          0
#define STATUS_SUCCESS       1

static unsigned short CsmUnlock()
{
	volatile unsigned short temp;

	/*
	 * Load the key registers with the current password. The 0xFFFF's are dummy
	 * passwords.  User should replace them with the correct password for the DSP.
	 */

	EALLOW;
		CsmRegs.KEY0 = 0xFFFF;
		CsmRegs.KEY1 = 0xFFFF;
		CsmRegs.KEY2 = 0xFFFF;
		CsmRegs.KEY3 = 0xFFFF;
		CsmRegs.KEY4 = 0xFFFF;
		CsmRegs.KEY5 = 0xFFFF;
		CsmRegs.KEY6 = 0xFFFF;
		CsmRegs.KEY7 = 0xFFFF;
	EDIS;

	/*
	 * Perform a dummy read of the password locations
	 * if they match the key values, the CSM will unlock
	 */

	temp = CsmPwl.PSWD0;
	temp = CsmPwl.PSWD1;
	temp = CsmPwl.PSWD2;
	temp = CsmPwl.PSWD3;
	temp = CsmPwl.PSWD4;
	temp = CsmPwl.PSWD5;
	temp = CsmPwl.PSWD6;
	temp = CsmPwl.PSWD7;

	/*
	 * If the CSM unlocked, return succes, otherwise return
	 * failure.
	 */
	if (CsmRegs.CSMSCR.bit.SECURE == 0) {
		return STATUS_SUCCESS;
	}

	return STATUS_FAIL;

}
#endif

/*---------------------------------------------------------------------------
 * Example: IntOsc1Sel:
 *---------------------------------------------------------------------------
 * This function switches to Internal Oscillator 1 and turns off all other clock
 * sources to minimize power consumption
 */
static void IntOsc1Sel (void)
{
	EALLOW;
//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF   = 0;
		*CLKCTL &= ~CLKCTL_INTOSC1OFF;

//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL = 0;	/* Clk Src = INTOSC1 */
		*CLKCTL &= ~CLKCTL_OSCCLKSRCSEL;

//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF    = 1;	/* Turn off XCLKIN   */
		*CLKCTL |= CLKCTL_XCLKINOFF;

//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF   = 1;	/* Turn off XTALOSC  */
		*CLKCTL |= CLKCTL_XTALOSCOFF;

//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF   = 1;	/* Turn off INTOSC2  */
		*CLKCTL |= CLKCTL_INTOSC2OFF;

	EDIS;
}

/*---------------------------------------------------------------------------
 * Example: IntOsc2Sel:
 *---------------------------------------------------------------------------
 * This function switches to Internal oscillator 2 from External Oscillator
 * and turns off all other clock sources to minimize power consumption
 * NOTE: If there is no external clock connection, when switching from
 *       INTOSC1 to INTOSC2, EXTOSC and XLCKIN must be turned OFF prior
 *       to switching to internal oscillator 1
 */
#if 0
static void IntOsc2Sel (void)
{
	EALLOW;
//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF    = 0;	/* Turn on INTOSC2                    */
		*CLKCTL &= ~CLKCTL_INTOSC2OFF;
//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRC2SEL = 1;	/* Switch to INTOSC2                  */
		*CLKCTL |= CLKCTL_OSCCLKSRC2SEL;
//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF     = 1;	/* Turn off XCLKIN                    */
		*CLKCTL |= CLKCTL_XCLKINOFF;
//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF    = 1;	/* Turn off XTALOSC                   */
		*CLKCTL |= CLKCTL_XTALOSCOFF;
//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL  = 1;	/* Switch to Internal Oscillator 2    */
		*CLKCTL |= CLKCTL_OSCCLKSRCSEL;
//		SysCtrlRegs.CLKCTL.bit.WDCLKSRCSEL   = 1;	/* Switch Watchdog Clk Src to INTOSC2 */
		*CLKCTL |= CLKCTL_WDCLKSRCSEL;
//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF    = 1;	/* Turn off INTOSC1                   */
		*CLKCTL |= CLKCTL_INTOSC1OFF;
	EDIS;
}
#endif
/*---------------------------------------------------------------------------
 * Example: XtalOscSel:
 *---------------------------------------------------------------------------
 * This function switches to External CRYSTAL oscillator and turns off all other clock
 * sources to minimize power consumption. This option may not be available on all
 * device packages
 */
#if 0
static void XtalOscSel (void)
{
	EALLOW;
//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF    = 0;	/* Turn on XTALOSC                           */
		*CLKCTL &= ~CLKCTL_XTALOSCOFF;
//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF     = 1;	/* Turn off XCLKIN                           */
		*CLKCTL |= CLKCTL_XCLKINOFF;
//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRC2SEL = 0;	/* Switch to external clock                  */
		*CLKCTL &= ~CLKCTL_OSCCLKSRC2SEL;
//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL  = 1;	/* Switch from INTOSC1 to INTOSC2/ext clk    */
		*CLKCTL |= CLKCTL_OSCCLKSRCSEL;
//		SysCtrlRegs.CLKCTL.bit.WDCLKSRCSEL   = 1;	/* Switch Watchdog Clk Src to external clock */
		*CLKCTL |= CLKCTL_WDCLKSRCSEL;
//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF    = 1;	/* Turn off INTOSC2                          */
		*CLKCTL |= CLKCTL_INTOSC2OFF;
//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF    = 1;	/* Turn off INTOSC1                          */
		*CLKCTL |= CLKCTL_INTOSC1OFF;
	EDIS;
}
#endif

/*---------------------------------------------------------------------------
 * Example: ExtOscSel:
 *
 * This function switches to External oscillator and turns off all other clock
 * sources to minimize power consumption.
 */
#if 0
static void ExtOscSel (void)
{
	EALLOW;
//		SysCtrlRegs.XCLK.bit.XCLKINSEL       = 1;	/* 1-GPIO19 = XCLKIN, 0-GPIO38 = XCLKIN      */
		*XCLK |= XCLK_XCLKINSEL;
//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF    = 1;	/* Turn on XTALOSC                           */
		*CLKCTL |= CLKCTL_XTALOSCOFF;
//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF     = 0;	/* Turn on XCLKIN                            */
		*CLKCTL &= ~CLKCTL_XCLKINOFF;
//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRC2SEL = 0;	/* Switch to external clock                  */
		*CLKCTL &= ~CLKCTL_OSCCLKSRC2SEL;
//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL  = 1;	/* Switch from INTOSC1 to INTOSC2/ext clk    */
		*CLKCTL |= CLKCTL_OSCCLKSRCSEL;
//		SysCtrlRegs.CLKCTL.bit.WDCLKSRCSEL   = 1;	/* Switch Watchdog Clk Src to external clock */
		*CLKCTL |= CLKCTL_WDCLKSRCSEL;
//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF    = 1;	/* Turn off INTOSC2                          */
		*CLKCTL |= CLKCTL_INTOSC2OFF;
//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF    = 1;	/* Turn off INTOSC1                          */
		*CLKCTL |= CLKCTL_INTOSC1OFF;
	EDIS;
}
#endif

#if 0
/*******************************************************************************
 *
 */
int dsp_init( void )
{

	// Disable the watchdog
//	DisableDog();
	if( watchdog_init( 0 ) ) {
		return -1;
	}

#if 1
	/* *IMPORTANT*
	 * The Device_cal function, which copies the ADC & oscillator calibration values
	 * from TI reserved OTP into the appropriate trim registers, occurs automatically
	 * in the Boot ROM. If the boot ROM code is bypassed during the debug process, the
	 * following function MUST be called for the ADC and oscillators to function according
	 * to specification. The clocks to the ADC MUST be enabled before calling this
	 * function.
	 * See the device data manual and/or the ADC Reference
	 * Manual for more information.
	 */
	EALLOW;

	//SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1; // Enable ADC peripheral clock
	* PCLKCR0 |= (1<<3);
	(*Device_cal)();
//	SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 0; // Return ADC clock to original state
	* PCLKCR0 &= ~(1<<3);

	EDIS;
#endif

	clk_init();
}

/******************************************************************************/
/* Clock control                                                              */
/******************************************************************************/

/*******************************************************************************
 *
 */
int clk_init( void )
{

#if 0
	// Select Internal Oscillator 1 as Clock Source (default), and turn off all unused clocks to
	// conserve power.
	IntOsc1Sel();
#endif

	// Initialize the PLL control: PLLCR and CLKINDIV
	// DSP28_PLLCR and DSP28_CLKINDIV are defined in DSP2803x_Examples.h
	InitPll(DSP28_PLLCR,DSP28_DIVSEL);


	/*
	 * Initialize the peripheral clocks
	 */
	EALLOW;
	*pCLKCR0 = PCLKCR0_INIT;
	*pCLKCR1 = PCLKCR1_INIT;
	*pCLKCR3 = PCLKCR3_INIT;

	EDIS;

	return 0;
}

/*******************************************************************************
 *
 */
int clk_ctrl( int clkID, int bEnable )
{
	int retVal;

	volatile unsigned short * pReg;
	int regVal;
	int intLockVal;

	switch( _CLK_ID_2_REG_NO( clkID ) ) {
	case 0 : pReg = PCLKCR0; break;
	case 1 : pReg = PCLKCR1; break;
	case 3 : pReg = PCLKCR3; break;
	default : pReg = NULL; break;
	}

	if( pReg ) {
		regVal = 1 << _CLK_ID_2_BIT( clkID );

		intLockVal = sysIntLock();

		EALLOW;
		if( bEnable ) {
			*pReg |= regVal;
		} else {
			*pReg &= ~regVal;
		}
		EDIS;
		sysIntUnlock( intLockVal );

		retVal = 0;
	}

	return retVal;
}

/******************************************************************************/
/* for Watchdog timer                                                         */
/******************************************************************************/

/*******************************************************************************
 *
 */
static int watchdog_init( int bEnable )
{
	int retVal = -1;

	if( bEnable ) {
		/*** Sorry : This version not support Watchdog-timer */
	} else {

		EALLOW;
//		SysCtrlRegs.WDCR= 0x0068;
		*WDCR = 0x0068;
		EDIS;

		retVal = 0;
	}

	return retVal;
}
#endif
/*** end of file : clkCtrl.c ***/
