/*** flashDrv.c - TMS320F28035 Flash Memory Driver ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/23 mimu
              First Version
 */

/*
 * <<< Flash API Checklist >>> (from : Flash2803x_API_usingCCS4.0v_Readme.pdf)
 *
 * Integration of the Flash API into user software requires that the system designer
 * implement operations to satisfy several key requirements. The following checklist
 * gives an overview of the steps required to use the API.
 * These steps are further discussed in detail in the reference section indicated.
 *
 * --- Before using the API, do the following:---
 * +------+-----------------------------------------------------------------------+-----------+-------+
 * | Step | Description                                                           | Reference | check |
 * +------+-----------------------------------------------------------------------+-----------+-------+
 * |   1  | Modify Flash2803x_API_Config.h for your target operating conditions.  | Section 6 | mimu  |
 * +------+-----------------------------------------------------------------------+-----------+-------+
 * |   2  | Include Flash2803x_API_Library.h in your source code.                 | Section 7 | mimu  |
 * +------+-----------------------------------------------------------------------+-----------+-------+
 * |   3  | Add the proper Flash API library to your project.                     |           |       |
 * |      | When using the Flash API, build your code with the large memory model.|           |       |
 * |      | The API Library is built in 28x Object code (OBJMODE = 1, AMODE = 1)  | Section 8 | mimu  |
 * +------+-----------------------------------------------------------------------+-----------+-------+
 *
 * In your application, before calling any Flash API functions do the following:
 *
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * | Step | Description                                                                       | Reference  | Check |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |   4  | Initialize the PLL control register (PLLCR) and wait for the PLL to lock.         | Section 9  | mimu  |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |   5  | Make sure the PLL is not running in limp mode. If the PLL is in limp mode, do     |            |       |
 * |      | not call any of the API functions as the device will not be running at the proper |            |       |
 * |      | frequency.                                                                        | Section 10 | mimu  |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |   6  | Optional: The API must execute from zero-wait state internal SARAM or ROM.        |            |       |
 * |      | If the API is to be copied from flash/OTP into internal SARAM memory then         |            |       |
 * |      | follow the instructions in this section.                                          | Section 11 |       | !! yet !!
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |   7  | Initialize the 32-bit global variable Flash_CPUScaleFactor                        | Section 12 | mimu  |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |   8  | Initialize the global function pointer Flash_CallbackPtr to point to the          |            |       |
 * |      | applicationfs callback function. Alternatively set the pointer to NULL.          | Section 13 | mimu  |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |   9  | Optional: Disable global interrupts before calling an API function.               | Section 14 |       | !! yet !!
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |  10  | Understand the API restrictions detailed in this section before making any API    |            |       |
 * |      | calls.                                                                            | Section 15 | mimu  |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |  11  | Optional: Run the frequency toggle test to confirm proper frequency               |            |       |
 * |      | configuration of the Flash API. Note: The ToggleTest function will execute        |            |       |
 * |      | forever. You must halt the processor to stop this test.                           | Section 16 | mimu  |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |  12  | Optional: Unlock the code security module (CSM).                                  | Section 17 |       |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 * |  13  | Call the Flash API Functions described in the API Reference                       | Section 18 |       |
 * +------+-----------------------------------------------------------------------------------+------------+-------+
 *
 * The called flash API function will do the following:
 *
 * @ The watchdog timer is disabled. (Section 15).
 * @  Performs the called operation and:
 *   + Disables and restores global interrupts (via INTM, DBGM, XNMICR) around time critical code
 *     segments. (Section 14).
 *   + Invokes the callback function if Flash_CallbackPtr is not NULL.
 * @ Returns success or an error code. These are defined in F2803x_API_Library.h (Section 18.11)
 *
 * The userfs code should then do the following:
 *
 * +------+--------------------------------------------------+---------------+-------+
 * | Step | Description                                      | Reference     | check |
 * +------+--------------------------------------------------+---------------+-------+
 * |  14  | Check the return status against the error codes. | Section 18.11 |       |
 * +------+--------------------------------------------------+---------------+-------+
 * |  15  | Optional: Re-enable the watchdog timer.          | ------------- |       |
 * +------+--------------------------------------------------+---------------+-------+
 */

/*** INCLUDE ****/
#include "config.h"
#include "sysLib.h"

#include "tms320f28035/tms320f28035.h"
#include "Flash2803x_API_Library.h"

#include "flashDrv.h"


/*** DEFINE ***/
//#define	DEBUG_FLASH		// force DEBUG
#define	FLASH_API_CALLBACK_ENABLE


/*** Register **/
#define	Flash_CPUScaleFactor	(volatile unsigned long *)0x000D04	/* for TI Flash API */
#define	Flash_CallbackPtr		(volatile unsigned long *)0x000D02			/* for TI Flash API */

#define	FOPT		(volatile unsigned short *)0x0A80		/* Flash Option Register					*/
#define	FPWR		(volatile unsigned short *)0x0A82		/* Flash Power Modes Register				*/
#define	FSTATUS		(volatile unsigned short *)0x0A83		/* Status Register							*/
#define	FSTDBYWAIT	(volatile unsigned short *)0x0A84		/* Flash Sleep To Standby Wait Register		*/
#define	FACTIVEWAIT	(volatile unsigned short *)0x0A85		/* Flash Standby To Active Wait Register	*/
#define	FBANKWAIT	(volatile unsigned short *)0x0A86		/* Flash Read Access Wait State Register	*/
#define	FOTPWAIT	(volatile unsigned short *)0x0A87		/* OTP Read Access Wait State Register		*/

/*** FOPT ***/
#define	FOPT_ENPIPE				(1<<0)		/* Enable Flash Pipeline Mode Bit. */

/*** FPWR ***/
#define	FPWR_PWR_SLEEP			(0)			/* Pump and bank sleep (lowest power)	*/
#define	FPWR_PWR_STANDBY		(1)			/* Pump and bank standby				*/
#define	FPWR_PWR_ACTIVE			(3)			/* Pump and bank active (highest power)	*/

/*** FSTATUS ***/
#define	FSTATUS_3VSTAT			(1<<8)		/* Flash Voltage (VDD3VFL) Status Latch Bit.		*/
#define	FSTATUS_ACTIVEWAITS		(1<<3)		/* Bank and Pump Standby To Active Wait Counter Status Bit.		*/
#define	FSTATUS_STDBYWAITS		(1<<2)		/* Bank and Pump Sleep To Standby Wait Counter Status Bit.		*/

#define	FSTATUS_PWRS(_reg)		((_reg)&0x03)
#define	FSTATUS_PWRS_SLEEP		(0)			/* Pump and bank in sleep mode (lowest power)				*/
#define	FSTATUS_PWRS_STANDBY	(1)			/* Pump and bank in standby mode							*/
#define	FSTATUS_PWRS_ACTIVE		(3)			/* Pump and bank active and in read mode (highest power)	*/

/*** FSTDBYWAIT ***/

/*** FACTIVEWAIT ***/

/*** FBANKWAIT ***/
#define	FBANKWAIT_PAGEWAIT(_val)	( ((_val)&0xf) << 8 )	/* Flash Paged Read Wait States.	*/
#define	FBANKWAIT_RANDWAIT(_val)	( ((_val)&0xf)      )	/* Flash Random Read Wait States.	*/

/*** FOTPWAIT ***/
#define	FOPTPWAIT_OTPWAIT(_val)		( ((_val)&0x1f)     )	/* OTP Read Wait States.			*/

/*** FUNCTION ***/

/*
 * InitFlash that will be run from RAM need to be assigned to
 * a different section.  This section will then be mapped to a load and
 * run address using the linker cmd file.
 */
#pragma CODE_SECTION(InitFlash, "ramfuncs");
static void InitFlash(void);

#ifdef	FLASH_API_CALLBACK_ENABLE
#pragma CODE_SECTION(flashApiCallback, "ramfuncs" );
static void flashApiCallback( void );
#endif	/* FLASH_API_CALLBACK_ENABLE */

/*** WORK ***/
/*
 * +------+---------+-------------------+------+-------------------------------+
 * | name | Sec No. | address           | size | Note                          |
 * +------+---------+-------------------+------+-------------------------------+
 * |   H  |    0    | 3e:8000 - 3e:9fff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   G  |    1    | 3e:a000 - 3e:bfff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   F  |    2    | 3e:c000 - 3e:dfff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   E  |    3    | 3e:e000 - 3e:ffff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   D  |    4    | 3f:0000 - 3f:1fff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   C  |    5    | 3f:2000 - 3f:3fff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   B  |    6    | 3f:4000 - 3f:5fff |  8K  |                               |
 * +------+---------+-------------------+------+-------------------------------+
 * |   A  |    7    | 3f:6000 - 3f:7f7f |  8K  |                               |
 * |      |         +-------------------+      +-------------------------------+
 * |      |         | 3f:7f80 - 3f:7ff5 |      | Used by Code Security Module  |
 * |      |         +-------------------+      +-------------------------------+
 * |      |         | 3f:7ff6 - 3f:7ff7 |      | Boot-to-Flash Entry Point     |
 * |      |         +-------------------+      +-------------------------------+
 * |      |         | 3f:7ff8 - 3f:7fff |      | Security Password(128bit)     |
 * +------+---------+-------------------+------+-------------------------------+
 */

static struct flash_sec_t {
	unsigned short * pTop;
	unsigned long size;
	unsigned short apiMask;
} flashSecTbl[FLASH_SEC_NUM] =
{
	{ (unsigned short *)0x3e8000, (8*1024), SECTORH,  }, /* 0 : FLASH_H */
	{ (unsigned short *)0x3ea000, (8*1024), SECTORG,  }, /* 1 : FLASH_H */
	{ (unsigned short *)0x3ec000, (8*1024), SECTORF,  }, /* 2 : FLASH_H */
	{ (unsigned short *)0x3ee000, (8*1024), SECTORE,  }, /* 3 : FLASH_H */
	{ (unsigned short *)0x3f0000, (8*1024), SECTORD,  }, /* 4 : FLASH_H */
	{ (unsigned short *)0x3f2000, (8*1024), SECTORC,  }, /* 5 : FLASH_H */
	{ (unsigned short *)0x3f4000, (8*1024), SECTORB,  }, /* 6 : FLASH_H */
	{ (unsigned short *)0x3f6000, (8*1024), SECTORA,  }, /* 7 : FLASH_H */
};

/*******************************************************************************
 *
 */
int flashInit( void )
{
	InitFlash();
	return 0;
}

/*******************************************************************************
 *
 */
unsigned long flashSecSize( unsigned int secNo )
{
	unsigned long retVal;

	retVal = 0;

	if( secNo < FLASH_SEC_NUM ) {
		retVal = flashSecTbl[secNo].size;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
unsigned short * flashSecTop( unsigned int secNo )
{
	if( secNo < FLASH_SEC_NUM ) {
		return flashSecTbl[secNo].pTop;
	}

	return 0;
}

/*******************************************************************************
 *
 */
int flashErase( unsigned int secNo, unsigned int secNum )
{
	int retVal;
	unsigned short mask;
	int i;
	FLASH_ST	flashStatus;
	int result;

	int intLockVal;

	retVal = -1;

	if( (secNo + secNum) < FLASH_SEC_NUM ) {

		intLockVal = sysIntLock();

		mask = 0;
		for( i=0; i<secNum; i++, secNo++ ) {
			mask |= flashSecTbl[secNo].apiMask;
		}
		result = Flash_Erase( mask, &flashStatus );
		if( result == STATUS_SUCCESS ) {
			retVal = 0;
		}

		sysIntUnlock( intLockVal );

	}


	return retVal;
}

/*******************************************************************************
 *
 */
int flashWrite( int secNo, unsigned long offset, unsigned short * pData, unsigned short size )
{
	int retVal;
	unsigned short * pFlash;
	FLASH_ST	flashStatus;
	int result;
	int intLockVal;

	retVal = -1;

	if( secNo < FLASH_SEC_NUM ) {

		intLockVal = sysIntLock();

		/* write */
		pFlash = flashSecTbl[secNo].pTop;
		pFlash += offset;
		result = Flash_Program( (Uint16 *)pFlash, (Uint16 *)pData, size, &flashStatus );

		if( result == STATUS_SUCCESS ) {

			/* verify */
			result = Flash_Verify( (Uint16 *)pFlash, (Uint16 *)pData, size, &flashStatus );
			if( result == STATUS_SUCCESS ) {
				retVal = 0;
			}
		}

		sysIntUnlock( intLockVal );
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int flashVerify( int secNo, unsigned long offset, unsigned short * pData, unsigned short size )
{
	int retVal;
	unsigned short * pFlash;
	FLASH_ST	flashStatus;
	int result;
	int intLockVal;

	retVal = -1;

	if( secNo < FLASH_SEC_NUM ) {

		intLockVal = sysIntLock();

		pFlash = flashSecTbl[secNo].pTop;
		pFlash += offset;
		result = Flash_Verify( (Uint16 *)pFlash, (Uint16 *)pData, size, &flashStatus );

		if( result == STATUS_SUCCESS ) {
			retVal = 0;
		}

		sysIntUnlock( intLockVal );
	}


	return retVal;
}

/*---------------------------------------------------------------------------
 * Example: InitFlash:
 *---------------------------------------------------------------------------
 * This function initializes the Flash Control registers
 *
 *                   CAUTION
 * This function MUST be executed out of RAM. Executing it
 * out of OTP/Flash will yield unpredictable results
 */
//#pragma DATA_SECTION( Flash_CPUScaleFactor, "FlashScalingVar" );
//Uint32 Flash_CPUScaleFactor;

//#pragma DATA_SECTION(Flash_CallbackPtr, "FlashCallbackVar");
//void (*Flash_CallbackPtr) (void);

static void InitFlash(void)
{
	EALLOW;

		*FPWR = FPWR_PWR_ACTIVE;
		*FSTATUS = FSTATUS_3VSTAT;

		/*
		 * Enable Flash Pipeline mode to improve performance
		 * of code executed from Flash.
		 */
//		FlashRegs.FOPT.bit.ENPIPE = 1;
		*FOPT = FOPT_ENPIPE;

		/*                CAUTION
		 * Minimum waitstates required for the flash operating
		 * at a given CPU rate must be characterized by TI.
		 * Refer to the datasheet for the latest information.
		 */

#if 0
		/*
		 * Set the Paged Waitstate for the Flash
		 */
		FlashRegs.FBANKWAIT.bit.PAGEWAIT = 2;

		/*
		 * Set the Random Waitstate for the Flash
		 */
		FlashRegs.FBANKWAIT.bit.RANDWAIT = 2;
#else
		*FBANKWAIT = FBANKWAIT_PAGEWAIT(2) | FBANKWAIT_RANDWAIT(2);

#endif

		/*
		 * Set the Waitstate for the OTP
		 */
//		FlashRegs.FOTPWAIT.bit.OTPWAIT = 2;
		*FOTPWAIT = FOPTPWAIT_OTPWAIT(2);

		/*                CAUTION
		 * ONLY THE DEFAULT VALUE FOR THESE 2 REGISTERS SHOULD BE USED
		 */
//		FlashRegs.FSTDBYWAIT.bit.STDBYWAIT = 0x01FF;
//		FlashRegs.FACTIVEWAIT.bit.ACTIVEWAIT = 0x01FF;
		*FSTDBYWAIT = 0x01ff;
		*FACTIVEWAIT = 0x01ff;

		/*
		 * for TI Flash API
		 */
		*Flash_CPUScaleFactor = SCALE_FACTOR;
#ifdef FLASH_API_CALLBACK_ENABLE
		*Flash_CallbackPtr = (unsigned long)flashApiCallback;
#else
		*Flash_CallbackPtr = 0;
#endif

	EDIS;

	/*
	 * Force a pipeline flush to ensure that the write to
	 * the last register configured occurs before returning.
	 */

	asm(" RPT #7 || NOP");
}

/*******************************************************************************
 * flashApiCallback()
 *
 * !! ATTATION !!
 *       Userfs application code to execute during the callback function
 *       This must not execute from flash/OTP or read data from flash/OTP
 */
#ifdef	FLASH_API_CALLBACK_ENABLE
static void flashApiCallback( void )
{
}
#endif	/* FLASH_API_CALLBACK_ENABLE */

/*******************************************************************************
 *
 */
#ifdef	DEBUG_FLASH
void flashDebug( void )
{
	int result;
	float32	apiVersion;
	Uint16	apiVersionHex;
	FLASH_ST	flashStatus;

	apiVersion = Flash_APIVersion();
	apiVersionHex = Flash_APIVersionHex();

#if 0
	/*
	 * use GPIO 12
	 */
	Flash_ToggleTest( (volatile unsigned long *)0x6FC6, (unsigned long)(1<<12) );
#endif


	result = Flash_Erase( SECTORH, &flashStatus );

}

#endif
