/*** gpioDrv.c - TMS320F28035(piccolo) GPIO Drvier ***/

/*** Copyright(C) 2010 Sansei system Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/09 mimu
              First Version
 */

/*** NOTE ***
 * Table 4-17. GPIO Registers
 * +--------------+---------+-------+------------------------------------------------------+
 * |              |         | SIZE  |                                                      |
 * | NAME         | ADDRESS | (x16) | DESCRIPTION                                          |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIO CONTROL REGISTERS (EALLOW PROTECTED)                                             |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPACTRL      | 0x6F80  |   2   | GPIO A Control Register (GPIO0 to 31)                |
 * | GPAQSEL1     | 0x6F82  |   2   | GPIO A Qualifier Select 1 Register (GPIO0 to 15)     |
 * | GPAQSEL2     | 0x6F84  |   2   | GPIO A Qualifier Select 2 Register (GPIO16 to 31)    |
 * | GPAMUX1      | 0x6F86  |   2   | GPIO A MUX 1 Register (GPIO0 to 15)                  |
 * | GPAMUX2      | 0x6F88  |   2   | GPIO A MUX 2 Register (GPIO16 to 31)                 |
 * | GPADIR       | 0x6F8A  |   2   | GPIO A Direction Register (GPIO0 to 31)              |
 * | GPAPUD       | 0x6F8C  |   2   | GPIO A Pull Up Disable Register (GPIO0 to 31)        |
 * | GPBCTRL      | 0x6F90  |   2   | GPIO B Control Register (GPIO32 to 44)               |
 * | GPBQSEL1     | 0x6F92  |   2   | GPIO B Qualifier Select 1 Register (GPIO32 to 44)    |
 * | GPBMUX1      | 0x6F96  |   2   | GPIO B MUX 1 Register (GPIO32 to 44)                 |
 * | GPBDIR       | 0x6F9A  |   2   | GPIO B Direction Register (GPIO32 to 44)             |
 * | GPBPUD       | 0x6F9C  |   2   | GPIO B Pull Up Disable Register (GPIO32 to 44)       |
 * | AIOMUX1      | 0x6FB6  |   2   | Analog, I/O mux 1 register (AIO0 to AIO15)           |
 * | AIODIR       | 0x6FBA  |   2   | Analog, I/O Direction Register (AIO0 to AIO15)       |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIO DATA REGISTERS (NOT EALLOW PROTECTED)                                            |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPADAT       | 0x6FC0  |   2   | GPIO A Data Register (GPIO0 to 31)                   |
 * | GPASET       | 0x6FC2  |   2   | GPIO A Data Set Register (GPIO0 to 31)               |
 * | GPACLEAR     | 0x6FC4  |   2   | GPIO A Data Clear Register (GPIO0 to 31)             |
 * | GPATOGGLE    | 0x6FC6  |   2   | GPIO A Data Toggle Register (GPIO0 to 31)            |
 * | GPBDAT       | 0x6FC8  |   2   | GPIO B Data Register (GPIO32 to 44)                  |
 * | GPBSET       | 0x6FCA  |   2   | GPIO B Data Set Register (GPIO32 to 44)              |
 * | GPBCLEAR     | 0x6FCC  |   2   | GPIO B Data Clear Register (GPIO32 to 44)            |
 * | GPBTOGGLE    | 0x6FCE  |   2   | GPIO B Data Toggle Register (GPIO32 to 44)           |
 * | AIODAT       | 0x6FD8  |   2   | Analog I/O Data Register (AIO0 to AIO15)             |
 * | AIOSET       | 0x6FDA  |   2   | Analog I/O Data Set Register (AIO0 to AIO15)         |
 * | AIOCLEAR     | 0x6FDC  |   2   | Analog I/O Data Clear Register (AIO0 to AIO15)       |
 * | AIOTOGGLE    | 0x6FDE  |   2   | Analog I/O Data Toggle Register (AIO0 to AIO15)      |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIO INTERRUPT AND LOW POWER MODES SELECT REGISTERS (EALLOW PROTECTED)                |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIOXINT1SEL | 0x6FE0  |   1   | XINT1 GPIO Input Select Register (GPIO0 to 31)       |
 * | GPIOXINT2SEL | 0x6FE1  |   1   | XINT2 GPIO Input Select Register (GPIO0 to 31)       |
 * | GPIOXINT3SEL | 0x6FE2  |   1   | XINT3 GPIO Input Select Register (GPIO0 to 31)       |
 * | GPIOLPMSEL   | 0x6FE8  |   2   | LPM GPIO Select Register (GPIO0 to 31)               |
 * +--------------+---------+-------+------------------------------------------------------+
 */

/*** INCLUDE ***/
//#include "DSP2803x_Device.h"
#include "tms320f28035.h"
#include "gpioDrv.h"

/*** DEFINE ***/
//#define	CHECK_EN

/* GPIO CONTROL REGISTERS (EALLOW PROTECTED) */
#define	GPACTRL			(volatile unsigned long *)0x6F80  /* GPIO A Control Register (GPIO0 to 31)				*/
#define	GPAQSEL1		(volatile unsigned long *)0x6F82  /* GPIO A Qualifier Select 1 Register (GPIO0 to 15)	*/
#define	GPAQSEL2		(volatile unsigned long *)0x6F84  /* GPIO A Qualifier Select 2 Register (GPIO16 to 31)	*/
#define	GPAMUX1			(volatile unsigned long *)0x6F86  /* GPIO A MUX 1 Register (GPIO0 to 15)				*/
#define	GPAMUX2			(volatile unsigned long *)0x6F88  /* GPIO A MUX 2 Register (GPIO16 to 31)				*/
#define	GPADIR			(volatile unsigned long *)0x6F8A  /* GPIO A Direction Register (GPIO0 to 31)			*/
#define	GPAPUD			(volatile unsigned long *)0x6F8C  /* GPIO A Pull Up Disable Register (GPIO0 to 31)		*/

#define	GPBCTRL			(volatile unsigned long *)0x6F90  /* GPIO B Control Register (GPIO32 to 44)				*/
#define	GPBQSEL1		(volatile unsigned long *)0x6F92  /* GPIO B Qualifier Select 1 Register (GPIO32 to 44)	*/
#define	GPBMUX1			(volatile unsigned long *)0x6F96  /* GPIO B MUX 1 Register (GPIO32 to 44)				*/
#define	GPBDIR			(volatile unsigned long *)0x6F9A  /* GPIO B Direction Register (GPIO32 to 44)			*/
#define	GPBPUD			(volatile unsigned long *)0x6F9C  /* GPIO B Pull Up Disable Register (GPIO32 to 44)		*/

#define	AIOMUX1			(volatile unsigned long *)0x6FB6  /* Analog, I/O mux 1 register (AIO0 to AIO15)			*/
#define	AIODIR			(volatile unsigned long *)0x6FBA  /* Analog, I/O Direction Register (AIO0 to AIO15)		*/

/* GPIO DATA REGISTERS (NOT EALLOW PROTECTED) */
#define	GPADAT			(volatile unsigned long *)0x6FC0  /* GPIO A Data Register (GPIO0 to 31)					*/
#define	GPASET			(volatile unsigned long *)0x6FC2  /* GPIO A Data Set Register (GPIO0 to 31)				*/
#define	GPACLEAR		(volatile unsigned long *)0x6FC4  /* GPIO A Data Clear Register (GPIO0 to 31)			*/
#define	GPATOGGLE		(volatile unsigned long *)0x6FC6  /* GPIO A Data Toggle Register (GPIO0 to 31)			*/

#define	GPBDAT			(volatile unsigned long *)0x6FC8  /* GPIO B Data Register (GPIO32 to 44)				*/
#define	GPBSET			(volatile unsigned long *)0x6FCA  /* GPIO B Data Set Register (GPIO32 to 44)			*/
#define	GPBCLEAR		(volatile unsigned long *)0x6FCC  /* GPIO B Data Clear Register (GPIO32 to 44)			*/
#define	GPBTOGGLE		(volatile unsigned long *)0x6FCE  /* GPIO B Data Toggle Register (GPIO32 to 44)			*/

#define	AIODAT			(volatile unsigned long *)0x6FD8  /* Analog I/O Data Register (AIO0 to AIO15)			*/
#define	AIOSET			(volatile unsigned long *)0x6FDA  /* Analog I/O Data Set Register (AIO0 to AIO15)		*/
#define	AIOCLEAR		(volatile unsigned long *)0x6FDC  /* Analog I/O Data Clear Register (AIO0 to AIO15)		*/
#define	AIOTOGGLE		(volatile unsigned long *)0x6FDE  /* Analog I/O Data Toggle Register (AIO0 to AIO15)	*/

/* GPIO INTERRUPT AND LOW POWER MODES SELECT REGISTERS (EALLOW PROTECTED) */
#define	GPIOXINT1SEL	(volatile unsigned short *)0x6FE0  /* XINT1 GPIO Input Select Register (GPIO0 to 31)	*/
#define	GPIOXINT2SEL	(volatile unsigned short *)0x6FE1  /* XINT2 GPIO Input Select Register (GPIO0 to 31)	*/
#define	GPIOXINT3SEL	(volatile unsigned short *)0x6FE2  /* XINT3 GPIO Input Select Register (GPIO0 to 31)	*/
#define	GPIOLPMSEL		(volatile unsigned long *)0x6FE8   /* LPM GPIO Select Register (GPIO0 to 31)			*/

/*** FUNCTION ***/

/*** WORK ***/
static struct gpio_tbl_t {
	volatile unsigned long * pDataReg;
	volatile unsigned long * pSetReg;
	volatile unsigned long * pClrReg;
	volatile unsigned long * pToggleReg;

	volatile unsigned long * pDirectionReg;

//	volatile unsigned long * pCtrlReg;
	volatile unsigned long * pPullUpDisableReg;

	volatile unsigned long * pQualifierSelectReg;
	volatile unsigned long * pMuxReg;

	int shift;
	int shift2;

}gpioTbl[GPIO_NUM];

#ifdef	CHECK_EN
static const int gpioMask[GPIO_NUM] =
{
#define	_GPIO_MODE_DEFAULT (GPIO_MODE_IN|GPIO_MODE_OUT|GPIO_MODE_PULLUP_DIS|GPIO_MODE_PULLUP|GPIO_MODE_QUALIFIER_0|GPIO_MODE_QUALIFIER_1|GPIO_MODE_QUALIFIER_2|GPIO_MODE_QUALIFIER_3)

	/*  0 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/*  1 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/*  2 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/*  3 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/*  4 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/*  5 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/*  6 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/*  7 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2                | _GPIO_MODE_DEFAULT ),
	/*  8 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/*  9 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2                | _GPIO_MODE_DEFAULT ),

	/* 10 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 11 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2                | _GPIO_MODE_DEFAULT ),
	/* 12 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 13 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 14 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 15 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 16 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 17 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 18 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 19 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),

	/* 20 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 21 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 22 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 23 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 24 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 25 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 26 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 27 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 28 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 29 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),

	/* 30 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/* 31 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/* 32 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 33 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 34 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 35 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
	/* 36 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
	/* 37 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
	/* 38 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
	/* 39 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),

	/* 40 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/* 41 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
	/* 42 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 43 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
	/* 44 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),

};
#endif

/*******************************************************************************
 *
 */
int gpioInit( void )
{
	int no;

	for( no=0; no<GPIO_NUM; no++ ) {
		if( no < 32 ) {
			gpioTbl[no].pDataReg   = GPADAT;
			gpioTbl[no].pSetReg    = GPASET;
			gpioTbl[no].pClrReg    = GPACLEAR;
			gpioTbl[no].pToggleReg = GPATOGGLE;

			gpioTbl[no].pDirectionReg = GPADIR;

//			gpioTbl[no].pCtrlReg   = GPACTRL;
			gpioTbl[no].pPullUpDisableReg = GPAPUD;

			if( no < 16 ) {
				gpioTbl[no].pQualifierSelectReg = GPAQSEL1;
				gpioTbl[no].pMuxReg = GPAMUX1;
				gpioTbl[no].shift2 = no<<1;
			} else {
				gpioTbl[no].pQualifierSelectReg = GPAQSEL2;
				gpioTbl[no].pMuxReg = GPAMUX2;
				gpioTbl[no].shift2 = (no-16)<<1;
			}
			gpioTbl[no].shift = no;

		} else {
			gpioTbl[no].pDataReg   = GPBDAT;
			gpioTbl[no].pSetReg    = GPBSET;
			gpioTbl[no].pClrReg    = GPBCLEAR;
			gpioTbl[no].pToggleReg = GPBTOGGLE;

			gpioTbl[no].pDirectionReg = GPBDIR;

//			gpioTbl[no].pCtrlReg   = GBACTRL;
			gpioTbl[no].pPullUpDisableReg = GPAPUD;

			gpioTbl[no].pQualifierSelectReg = GPBQSEL1;
			gpioTbl[no].pMuxReg = GPBMUX1;
			gpioTbl[no].shift2 = (no-32)<<1;

			gpioTbl[no].shift = (no-32);
		}
	}

	return 0;
}


/*******************************************************************************
 *
 */
int gpioConfig( int no, int mode )
{
	unsigned long mask, mask2;
	unsigned long regVal;

	unsigned long mux;
	unsigned long dir;
	unsigned long pullup;
	unsigned long qualifier;

#ifdef	CHECK_EN
	if( no >= GPIO_NUM ) {
		return -1;
	}

	if( mode != (mode & gpioMask[no]) ) {
		return -1;
	}
#endif

	mask = ~(1<<no);
	mask2 = ~(3<<gpioTbl[no].shift2);

	switch( mode & (GPIO_MODE_GPIO|GPIO_MODE_1|GPIO_MODE_2|GPIO_MODE_3) ) {
	case GPIO_MODE_GPIO: mux = 0; break;
	case GPIO_MODE_1:    mux = 1; break;
	case GPIO_MODE_2:    mux = 2; break;
	case GPIO_MODE_3:    mux = 3; break;
	default: return -1;
	}

	switch( mode & (GPIO_MODE_IN|GPIO_MODE_OUT) ) {
	case GPIO_MODE_IN:  dir = 0; break;
	case GPIO_MODE_OUT: dir = 1; break;
	default: return -1;
	}

	switch( mode & (GPIO_MODE_PULLUP_DIS|GPIO_MODE_PULLUP) ) {
	case GPIO_MODE_PULLUP_DIS: pullup = 1; break;
	case 0: /* break; */
	case GPIO_MODE_PULLUP:     pullup = 0; break;
	default: return -1;
	}


	switch( mode & (GPIO_MODE_QUALIFIER_0|GPIO_MODE_QUALIFIER_1|GPIO_MODE_QUALIFIER_2|GPIO_MODE_QUALIFIER_3) ) {
	case 0:	/* break; */
	case GPIO_MODE_QUALIFIER_0: qualifier = 0; break;
	case GPIO_MODE_QUALIFIER_1: qualifier = 1; break;
	case GPIO_MODE_QUALIFIER_2: qualifier = 2; break;
	case GPIO_MODE_QUALIFIER_3: qualifier = 3; break;
	default : return -1;
	}

	dir    <<= gpioTbl[no].shift;
	pullup <<= gpioTbl[no].shift;

	qualifier <<= gpioTbl[no].shift2;
	mux       <<= gpioTbl[no].shift2;

	EALLOW;

	regVal = *gpioTbl[no].pDirectionReg;
	regVal &= mask;
	regVal |= dir;
	*gpioTbl[no].pDirectionReg = regVal;

	regVal = *gpioTbl[no].pPullUpDisableReg;
	regVal &= mask;
	regVal |= pullup;
	*gpioTbl[no].pPullUpDisableReg = regVal;

	regVal = *gpioTbl[no].pQualifierSelectReg;
	regVal &= mask2;
	regVal |= qualifier;
	*gpioTbl[no].pQualifierSelectReg = regVal;

	regVal = *gpioTbl[no].pMuxReg;
	regVal &= mask2;
	regVal |= mux;
	*gpioTbl[no].pMuxReg = regVal;

	EDIS;

	return 0;
}


/*******************************************************************************
 *
 */
int gpioControl( int no, int bHigh )
{
	if( bHigh ) {
		return gpioSet( no );
	}

	return gpioClear( no );

}

/*******************************************************************************
 *
 */
int gpioSet( int no )
{

#ifdef	CHECK_EN
	if( no >= GPIO_NUM ) {
		return -1;
	}
#endif

	*gpioTbl[no].pSetReg = (1<<gpioTbl[no].shift);

	return 0;
}

/*******************************************************************************
 *
 */
int gpioClear( int no )
{

#ifdef	CHECK_EN
	if( no >= GPIO_NUM ) {
		return -1;
	}
#endif

	*gpioTbl[no].pClrReg = (1<<gpioTbl[no].shift);

	return 0;
}

/*******************************************************************************
 *
 */
int gpioToggle( int no )
{

#ifdef	CHECK_EN
	if( no >= GPIO_NUM ) {
		return -1;
	}
#endif

	*gpioTbl[no].pToggleReg = (1<<gpioTbl[no].shift);

	return 0;
}

/*******************************************************************************
 *
 */
int gpioGet( int no )
{
	unsigned long regVal;

	regVal = *gpioTbl[no].pDataReg;
	if( regVal & (1<<gpioTbl[no].shift) ) {
		return 1;
	}

	return 0;
}

/*******************************************************************************
 *
 */
int gpioCtrlRegSet( int reg, int _value )
{
	volatile unsigned long * pReg;
	int shift;
	unsigned long mask;
	unsigned long value;
	unsigned long regVal;

	mask = 0x0ff;
	value = (unsigned long)_value;
	value &= mask;
	switch( reg ) {
	case GPIO_CTRLREG_QUALPRD_0_7:			/* GPIO 0-7 */
		pReg = GPACTRL;
		shift = 0;
		break;
	case GPIO_CTRLREG_QUALPRD_8_15:			/* GPIO 8-15 */
		pReg = GPACTRL;
		shift = 8;
		break;
	case GPIO_CTRLREG_QUALPRD_16_23:			/* GPIO 16-23 */
		pReg = GPACTRL;
		shift = 16;
		break;
	case GPIO_CTRLREG_QUALPRD_24_31:			/* GPIO 24-31 */
		pReg = GPACTRL;
		shift = 24;
		break;
	case GPIO_CTRLREG_QUBLPRD_32_39:			/* GPIO 32-39 */
		pReg = GPBCTRL;
		shift = 0;
		break;
	case GPIO_CTRLREG_QUBLPRD_40_44:			/* GPIO 32-44 */
		pReg = GPBCTRL;
		shift = 8;
	default:
		return -1;
	}

	mask  <<= shift;
	value <<= shift;

	mask =~mask;

	EALLOW;

	regVal = *pReg;
	regVal &= mask;
	regVal |= value;
	*pReg = regVal;

	EDIS;

	return 0;
}

/*** end of file : gpioDrv.c ***/
