/*** sciDrv.c - TMS320F28035 (piccolo) SCI Driver ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/15 mimu
              First Version
 */

/*** NOTE ***
 *
 *** ATTATION ***
 * 1) This driver support Channel A (channel 0) only
 *
 *** Register map ***
 * --- A Channel ---
 * SCICCR   0x0000-7050 1 SCI-A Communications Control Register
 * SCICTL1  0x0000-7051 1 SCI-A Control Register 1
 * SCIHBAUD 0x0000-7052 1 SCI-A Baud Register, High Bits
 * SCILBAUD 0x0000-7053 1 SCI-A Baud Register, Low Bits
 * SCICTL2  0x0000-7054 1 SCI-A Control Register 2
 * SCIRXST  0x0000-7055 1 SCI-A Receive Status Register
 * SCIRXEMU 0x0000-7056 1 SCI-A Receive Emulation Data Buffer Register
 * SCIRXBUF 0x0000-7057 1 SCI-A Receive Data Buffer Register

 * SCITXBUF 0x0000-7059 1 SCI-A Transmit Data Buffer Register
 * SCIFFTX  0x0000-705A 1 SCI-A FIFO Transmit Register
 * SCIFFRX  0x0000-705B 1 SCI-A FIFO Receive Register
 * SCIFFCT  0x0000-705C 1 SCI-A FIFO Control Register

 * SCIPRI   0x0000-705F 1 SCI-A Priority Control Register
 */

/*** INCLUDE ***/
#include "config.h"
#include "sysLib.h"

#include "gpioDrv.h"
#include "lib/ringLib.h"
#include "sciDrv.h"


/*** DEFINE ***/
#define	TX_BUFF_SIZE	32
#define	RX_BUFF_SIZE	64

#define	DEFAULT_BPS		9600UL
#define	DEFAULT_CHARLEN	8
#define	DEFAULT_STOP	1
#define	DEFAULT_PARITY	'N'

#define	TX_FIFO_LVL		1
#define	RX_FIFO_LVL		1

#define	TX_FIFO_NUM		4
#define	RX_FIFO_NUM		4

#ifndef	SUPPORT_SCI_A
#error
#endif

#ifndef	GPIO_NO_SCIRXDA
#error
#endif

#ifndef	GPIO_NO_SCITXDA
#error
#endif

typedef struct sci_reg_t {
/** **/
	volatile unsigned short	ccr;	/* Communications Control			*/
#	define	CCR_STOP_BITS_1	(0<<7)
#	define	CCR_STOP_BITS_2	(1<<7)

#	define	_CCR_EVEN_PARITY	(1<<6)
#	define	_CCR_ODD_PARITY	(0<<6)
#	define	_CCR_PARITY_ENABLE	(1<<5)
#	define	CCR_PARITY_EVEN		(_CCR_PARITY_ENABLE|_CCR_EVEN_PARITY)
#	define	CCR_PARITY_ODD		(_CCR_PARITY_ENABLE|_CCR_ODD_PARITY)
#	define	CCR_PARITY_NONE		0

#	define	CCR_LOOPBACK_ENA	(1<<4)

									/* ADDR/IDLE MODE SCI multiprocessor mode control bit */
#	define	CCR_ADDR_MODE		(1<<3)
#	define	CCR_IDLE_MODE		(0<<3)

#	define	CCR_CHAR(_len)		((_len)-1)	/* 1<=_len<=8 */

/** **/
	volatile unsigned short	ctl1;	/* Control 1						*/
#	define	CTL1_RX_ERRINT_ENA	(1<<6)
//#	define	CTL1_SW_RESET		(1<<5)
#	define	CTL1_SCI_RUN		(1<<5)
#	define	CTL1_TXWAKE			(1<<3)
#	define	CTL1_SLEEP			(1<<2)

#	define	CTL1_TXENA			(1<<1)
#	define	CTL1_RXENA			(1<<0)


	volatile unsigned short	hbaud;	/* Baud High Bits					*/
	volatile unsigned short	baud;	/* Baud Low Bits					*/
	volatile unsigned short	ctl2;	/* Control 2						*/
#	define	CTL2_TXRDY		(1<<7)
#	define	CTL2_TXEMPTY	(1<<6)
#	define	CTL2_RXINT_ENA	(1<<1)
#	define	CTL2_TXINT_ENA	(1<<0)

	volatile unsigned short	rxst;	/* Receive Status					*/
#	define	RXST_RXERROR	(1<<7)
#	define	RXST_RXRDY		(1<<6)
#	define	RXST_BRKDT		(1<<5)
#	define	RXST_FE			(1<<4)
#	define	RXST_OE			(1<<3)
#	define	RXST_PE			(1<<2)
#	define	RXST_RXWAKE		(1<<1)

	volatile unsigned short	rxemu;	/* Receive Emulation Data Buffer	*/
	volatile unsigned short	rxbuf;	/* Receive Data Buffer				*/
#	define	RXBUF_SCIFFFE	(1<<15)
#	define	RXBUF_SCIFFPE	(1<<14)
#	define	RXBUF_RXDT		(0xff)

	         unsigned short __dummy__0;

	volatile unsigned short	txbuf;	/* Transmit Data Buffer				*/
	volatile unsigned short	fftx;	/* FIFO Transmit					*/
//#define	FFTX_SCIRST		(1<<15)
#define	FFTX_SCIRUN			(1<<15)
#define	FFTX_SCIFFENA	(1<<14)
//#define	FFTX_TXFIFO_RESET	(1<<13)
#define	FFTX_TXFIFO_ENABLE	(1<<13)
#define	FFTX_TXFFST(_reg)	(((_reg)>>8)&0x01f)
#define	FFTX_TXFFINT_FLAG	(1<<7)
#define	FFTX_TXFFINT_CLR	(1<<6)
#define	FFTX_TXFFIENA		(1<<5)
#define	FFTX_TXFFIL(_lvl)	((_lvl)&0x1f)

	volatile unsigned short	ffrx;	/* FIFO Receive						*/
#define	FFRX_RXFFOVF		(1<<15)
#define	FFRX_RXFFOVF_CLR	(1<<14)
//#define	FFRX_RXFIFO_RESET	(1<<13)
#define	FFRX_RXFIFO_ENABLE	(1<<13)
#define	FFRX_RXFFST(_reg)	(((_reg)>>8)&0x01f)
#define	FFRX_RXFFINT		(1<<7)
#define	FFRX_RXFFINT_CLR	(1<<6)
#define	FFRX_RXFFIENA		(1<<5)
#define	FFRX_RXFFIL(_lvl)	((_lvl)&0x1f)

	volatile unsigned short	ffct;	/* FIFO Control						*/
#define	FFCT_ABD			(1<<15)
#define	FFCT_ABD_CLR		(1<<14)
#define	FFCT_CDC			(1<<13)
#define	FFCT_FFTXDLY(_dly)	((_dly)&0x0ff)

	         unsigned short __dummy__1;

	volatile unsigned short	pri;	/* Priority Control					*/
#define	PRI_SOFT		(1<<4)
#define	PRI_FREE		(1<<3)

} SciReg;

typedef struct sci_chan_t {
	int			st;
#define	ST_INIT		(1<<0)
#define	ST_CONFIG	(1<<1)
#define	ST_OPEN		(1<<2)

	SciReg *	pReg;

	unsigned long	bps;
	int				charLen;
	int				stop;
	char			parity;

	IntID		rxIntID;
	IntID		txIntID;

	Ring txRing;
	Ring rxRing;

	unsigned char txBuff[TX_BUFF_SIZE];
	unsigned char rxBuff[RX_BUFF_SIZE];


} SciChan;

/*** FUNCTION ***/
static int sciConfig( SciChan * pChan );
static void sciIntTx( void * arg );
static void sciIntRx( void * arg );

/*** WORK ***/
static unsigned long baseClock;
static SciChan sciChan;

/*******************************************************************************
 *
 */
int sciDrvInit( unsigned long _baseClock )
{
	SciReg * pReg;


	if( _baseClock == 0 ) {
		return -1;
	}

	/*
	 * Configuration GPIO for SCI
	 */
	{
		int mode;
		mode = GPIO_MODE_IN
		     | GPIO_MODE_PULLUP
#if	(GPIO_NO_SCIRXDA==7)
		     | GPIO_MODE_2;
#elif (GPIO_NO_SCIRXDA==28)
		     | GPIO_MODE_1;
#else
#error
#endif
		if( gpioConfig( GPIO_NO_SCIRXDA, mode ) ) {
			return -1;
		}

		mode = GPIO_MODE_OUT
		     | GPIO_MODE_PULLUP
#if (GPIO_NO_SCITXDA==12)
		     | GPIO_MODE_2;
#elif (GPIO_NO_SCITXDA==29)
		     | GPIO_MODE_1;
#else
#error
#endif

		if( gpioConfig( GPIO_NO_SCITXDA, mode ) ) {
			return -1;
		}
	}

	baseClock = _baseClock;

	pReg = (SciReg *)0x7050;
	sciChan.rxIntID = IntID_SciA_RX;
	sciChan.txIntID = IntID_SciA_TX;

	sciChan.txRing.pDataTop = &sciChan.txBuff[0];
	sciChan.txRing.size     = TX_BUFF_SIZE;
	if( ringInit( &sciChan.txRing ) ) {
		return -1;
	}

	sciChan.rxRing.pDataTop = &sciChan.rxBuff[0];
	sciChan.rxRing.size     = RX_BUFF_SIZE;
	if( ringInit( &sciChan.rxRing ) ) {
		return -1;
	}

	sciChan.pReg    = pReg;
	sciChan.bps     = DEFAULT_BPS;
	sciChan.charLen = DEFAULT_CHARLEN;
	sciChan.stop    = DEFAULT_STOP;
	sciChan.parity  = DEFAULT_PARITY;

	sciChan.st = ST_INIT;

	pReg->ctl1 = 0;	/* force reset */

	if( sysIntConnect( sciChan.rxIntID, sciIntRx, &sciChan ) ) {
		return -1;
	}
//	if( sysIntEnable( sciChan.rxIntID ) ) {
//		return -1;
//	}

	if( sysIntConnect( sciChan.txIntID, sciIntTx, &sciChan ) ) {
		return -1;
	}
//	if( sysIntEnable( sciChan.txIntID ) ) {
//		return -1;
//	}

	if( sciConfig( &sciChan ) ) {
		return -1;
	}

	return 0;
}

/*******************************************************************************
 *
 */
static int sciConfig( SciChan * pChan )
{
	SciReg * pReg;

	unsigned short ccrVal;
	unsigned long brrVal;
	unsigned short ffctVal;

	if( pChan->st & ST_OPEN ) {
		return -1;
	}

	pChan->st &= ~ST_CONFIG;

	ccrVal = 0;
	ffctVal = FFCT_ABD_CLR;

	switch( pChan->stop ) {
	case 1: ccrVal |= CCR_STOP_BITS_1; ffctVal = FFCT_FFTXDLY(0); break;
	case 2: ccrVal |= CCR_STOP_BITS_2; ffctVal = FFCT_FFTXDLY(1); break;
	default: return -1; /* break; */
	}

	switch( pChan->parity ) {
	case 'N': ccrVal |= CCR_PARITY_NONE; break;
	case 'E': ccrVal |= CCR_PARITY_EVEN; break;
	case 'O': ccrVal |= CCR_PARITY_ODD;  break;
	default: return -1; /* break; */
	}

	switch( pChan->charLen ) {
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
		ccrVal |= CCR_CHAR(pChan->charLen);
		break;
	default:
		return -1;
		/* break; */
	}

	{
//		unsigned long brr;
		unsigned long clk;
		unsigned long bps;
		unsigned long bps16[3];
		unsigned long d;
		unsigned long dMin;
		int i;
		int iMin;

		brrVal = ( baseClock/(pChan->bps * 8) ) - 1;
		if( (brrVal==0) || (0x0fffe<brrVal) ) {
			return -1;
		}
		clk = baseClock * 2;

		bps16[0] = clk/brrVal;
		bps16[1] = clk/(brrVal+1);
		bps16[2] = clk/(brrVal+2);

		bps = pChan->bps * 16;

		dMin = 0x7fffffff;
		iMin = -1;
		for( i=0; i<3; i++ ) {
			if( bps16[i] > bps ) {
				d = bps16[i] - bps;
			} else {
				d = bps - bps16[i];
			}
			if( d <= dMin ) {
				dMin = d;
				iMin = i;
			}
		}

		if( iMin < 0 ) {
			return -1;
		}

		brrVal += iMin;
		brrVal --;
	}

/// fftx <- e040  FFTX_SCIRUN | FFTX_SCIFFENA | FFTX_TXFIFO_ENABLE | FFTX_TXFFINT_CLR | FFTX_TXFFIL(0)
/// ffrx <- 2044  FFRX_RXFIFO_ENABLE | FFRX_RXFFINT_CLR | FFRX_RXFFIL(4)
/// ffct <- 0000
	pReg = pChan->pReg;

	pReg->ctl1 = 0;				/* force reset */
	pReg->ctl1 = CTL1_SCI_RUN;	/* device run, but Tx & Rx Stoped */

//	pReg->fftx = FFTX_TXFFINT_CLR;	/* force reset (TxFIFO), clear Interrut Flag */
//	pReg->fftx = FFTX_SCIRUN | FFTX_SCIFFENA | FFTX_TXFIFO_ENABLE | FFTX_TXFFIENA | FFTX_TXFFIL(TX_FIFO_LVL);
	pReg->fftx = FFTX_SCIRUN | FFTX_SCIFFENA | FFTX_TXFIFO_ENABLE | FFTX_TXFFIENA | FFTX_TXFFINT_CLR | FFTX_TXFFIL(TX_FIFO_LVL);

//	pReg->ffrx = FFRX_RXFFINT_CLR | FFRX_RXFFOVF_CLR;				/* force reset (RxFIFO) */
//	pReg->ffrx = FFRX_RXFIFO_ENABLE | FFRX_RXFFIL(RX_FIFO_LVL);
	pReg->ffrx = FFRX_RXFIFO_ENABLE | FFRX_RXFFINT_CLR | FFRX_RXFFIENA | FFRX_RXFFOVF_CLR | FFRX_RXFFIL(RX_FIFO_LVL);

	pReg->ffct = ffctVal;

	pReg->pri = PRI_FREE;

	pReg->ccr = ccrVal;
	pReg->hbaud = (unsigned short)( (brrVal >> 8) & 0xff );
	pReg->baud  = (unsigned short)( brrVal & 0xff );

//	pReg->ctl2 = CTL2_RXINT_ENA | CTL2_TXINT_ENA;

//	pReg->ctl1 = CTL1_SCI_RUN | CTL1_TXENA | CTL1_RXENA;

	pChan->st |= ST_CONFIG;

	return 0;
}

/*******************************************************************************
 *
 */
int sciCtrl( unsigned long bps, int charLen, char parity, int stop )
{
	int retVal = -1;

	if( (sciChan.st & (ST_INIT|ST_OPEN)) == ST_INIT ) {
		sciChan.bps = bps;
		sciChan.charLen = charLen;
		sciChan.parity = parity;
		sciChan.stop = stop;

		retVal = sciConfig( &sciChan );
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int sciOpen( void )
{
	int retVal = -1;

	if( (sciChan.st & (ST_INIT|ST_CONFIG|ST_OPEN)) == (ST_INIT|ST_CONFIG) ) {
		sciChan.st |= ST_OPEN;

		sciChan.pReg->ctl2 = CTL2_RXINT_ENA | CTL2_TXINT_ENA;
		sciChan.pReg->ctl1 = CTL1_SCI_RUN | CTL1_TXENA | CTL1_RXENA;
		sysIntEnable( sciChan.rxIntID );
		retVal = 0;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int sciClose( void )
{
	int retVal = -1;

	if( sciChan.st == (ST_INIT|ST_CONFIG|ST_OPEN) ) {
		sciChan.st = ST_INIT | ST_CONFIG;
		sciChan.pReg->ctl2 = CTL2_RXINT_ENA | CTL2_TXINT_ENA;
		sciChan.pReg->ctl1 = CTL1_SCI_RUN | CTL1_TXENA;
		retVal = 0;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int sciTx( unsigned char * pData, int len, int lenMin )
{
	int retVal;
	int setNum;

	retVal = 0;

	for( ; len > 0; ) {
		/* push to buffer */
		setNum = ringSet( &sciChan.txRing, pData, len );
		if( setNum < 0 ) {
			return -1;
		}

		retVal += setNum;
		pData += setNum;
		len -= setNum;
		lenMin -= setNum;

		/* kick transmitter */
		sysIntEnable( sciChan.txIntID );

		if( setNum == 0 ) {
			if( lenMin <= 0 ) {
				return retVal;
			}
		}
	}

	return retVal;
}

/*******************************************************************************
 *
 */
static void sciIntTx( void * arg )
{
	SciChan * pChan;
	SciReg  * pReg;

	unsigned char txBuff[TX_FIFO_NUM];
	unsigned short fftxVal;
	int txByte;
	int i;

	pChan = (SciChan *)arg;
	pReg = pChan->pReg;

	fftxVal = pReg->fftx;
	txByte = TX_FIFO_NUM - FFTX_TXFFST(fftxVal);

	txByte = ringGet( &pChan->txRing, &txBuff[0], txByte );

	if( txByte > 0 ) {
		for( i=0; i<txByte; i++ ) {
			pReg->txbuf = (unsigned short)txBuff[i];
		}
//		pReg->fftx |= FFTX_TXFFINT_CLR;
	} else {
		sysIntDisable( pChan->txIntID );
	}
	pReg->fftx |= FFTX_TXFFINT_CLR;
}

/*******************************************************************************
 *
 */
int sciRx( unsigned char * pData, int len, int lenMin )
{
	int retVal;
	int getNum;

	retVal = 0;

	for( ; len>0; ) {
		/* get from buffer */
		getNum = ringGet( &sciChan.rxRing, pData, len );
		if( getNum < 0 ) {
			return -1;
		}

		retVal += getNum;
		pData  += getNum;
		len    -= getNum;
		lenMin -= getNum;

		if( getNum == 0 ) {
			if( lenMin <= 0 ) {
				return retVal;
			}
		}
	}

	return retVal;
}

/*******************************************************************************
 *
 */
static void sciIntRx( void * arg )
{
	SciChan * pChan;
	SciReg  * pReg;

	unsigned char rxBuff[RX_FIFO_NUM];
	unsigned short ffrxVal;
	int rxByte;
	int i;

	pChan = (SciChan *)arg;
	pReg  = pChan->pReg;

	ffrxVal = pReg->ffrx;
	rxByte = FFRX_RXFFST( ffrxVal );

	for( ; rxByte; ) {
		if( rxByte > RX_FIFO_NUM ) {
			rxByte = RX_FIFO_NUM;
		}
		for( i=0; i<rxByte; i++ ) {
			rxBuff[i] = pReg->rxbuf;
		}
		ringSet( &pChan->rxRing, &rxBuff[0], rxByte );

		ffrxVal = pReg->ffrx;
		rxByte = FFRX_RXFFST( ffrxVal );
	}

	pReg->ffrx |= FFRX_RXFFINT_CLR;
}


/*** end of file : sciDrv.c ***/
