/*** gpioDrv.c - TMS320F28035(piccolo) GPIO Drvier ***/

/*** Copyright(C) 2010 Sansei system Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/09 mimu
              First Version
 */

/*** NOTE ***
 * Table 4-17. GPIO Registers
 * +--------------+---------+-------+------------------------------------------------------+
 * |              |         | SIZE  |                                                      |
 * | NAME         | ADDRESS | (x16) | DESCRIPTION                                          |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIO CONTROL REGISTERS (EALLOW PROTECTED)                                             |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPACTRL      | 0x6F80  |   2   | GPIO A Control Register (GPIO0 to 31)                |
 * | GPAQSEL1     | 0x6F82  |   2   | GPIO A Qualifier Select 1 Register (GPIO0 to 15)     |
 * | GPAQSEL2     | 0x6F84  |   2   | GPIO A Qualifier Select 2 Register (GPIO16 to 31)    |
 * | GPAMUX1      | 0x6F86  |   2   | GPIO A MUX 1 Register (GPIO0 to 15)                  |
 * | GPAMUX2      | 0x6F88  |   2   | GPIO A MUX 2 Register (GPIO16 to 31)                 |
 * | GPADIR       | 0x6F8A  |   2   | GPIO A Direction Register (GPIO0 to 31)              |
 * | GPAPUD       | 0x6F8C  |   2   | GPIO A Pull Up Disable Register (GPIO0 to 31)        |
 * | GPBCTRL      | 0x6F90  |   2   | GPIO B Control Register (GPIO32 to 44)               |
 * | GPBQSEL1     | 0x6F92  |   2   | GPIO B Qualifier Select 1 Register (GPIO32 to 44)    |
 * | GPBMUX1      | 0x6F96  |   2   | GPIO B MUX 1 Register (GPIO32 to 44)                 |
 * | GPBDIR       | 0x6F9A  |   2   | GPIO B Direction Register (GPIO32 to 44)             |
 * | GPBPUD       | 0x6F9C  |   2   | GPIO B Pull Up Disable Register (GPIO32 to 44)       |
 * | AIOMUX1      | 0x6FB6  |   2   | Analog, I/O mux 1 register (AIO0 to AIO15)           |
 * | AIODIR       | 0x6FBA  |   2   | Analog, I/O Direction Register (AIO0 to AIO15)       |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIO DATA REGISTERS (NOT EALLOW PROTECTED)                                            |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPADAT       | 0x6FC0  |   2   | GPIO A Data Register (GPIO0 to 31)                   |
 * | GPASET       | 0x6FC2  |   2   | GPIO A Data Set Register (GPIO0 to 31)               |
 * | GPACLEAR     | 0x6FC4  |   2   | GPIO A Data Clear Register (GPIO0 to 31)             |
 * | GPATOGGLE    | 0x6FC6  |   2   | GPIO A Data Toggle Register (GPIO0 to 31)            |
 * | GPBDAT       | 0x6FC8  |   2   | GPIO B Data Register (GPIO32 to 44)                  |
 * | GPBSET       | 0x6FCA  |   2   | GPIO B Data Set Register (GPIO32 to 44)              |
 * | GPBCLEAR     | 0x6FCC  |   2   | GPIO B Data Clear Register (GPIO32 to 44)            |
 * | GPBTOGGLE    | 0x6FCE  |   2   | GPIO B Data Toggle Register (GPIO32 to 44)           |
 * | AIODAT       | 0x6FD8  |   2   | Analog I/O Data Register (AIO0 to AIO15)             |
 * | AIOSET       | 0x6FDA  |   2   | Analog I/O Data Set Register (AIO0 to AIO15)         |
 * | AIOCLEAR     | 0x6FDC  |   2   | Analog I/O Data Clear Register (AIO0 to AIO15)       |
 * | AIOTOGGLE    | 0x6FDE  |   2   | Analog I/O Data Toggle Register (AIO0 to AIO15)      |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIO INTERRUPT AND LOW POWER MODES SELECT REGISTERS (EALLOW PROTECTED)                |
 * +--------------+---------+-------+------------------------------------------------------+
 * | GPIOXINT1SEL | 0x6FE0  |   1   | XINT1 GPIO Input Select Register (GPIO0 to 31)       |
 * | GPIOXINT2SEL | 0x6FE1  |   1   | XINT2 GPIO Input Select Register (GPIO0 to 31)       |
 * | GPIOXINT3SEL | 0x6FE2  |   1   | XINT3 GPIO Input Select Register (GPIO0 to 31)       |
 * | GPIOLPMSEL   | 0x6FE8  |   2   | LPM GPIO Select Register (GPIO0 to 31)               |
 * +--------------+---------+-------+------------------------------------------------------+
 */

/*** INCLUDE ***/
#include "config.h"
#include "sysLib.h"
//#include "DSP2803x_Device.h"
#include "tms320f28035.h"
#include "gpioDrv.h"

/*** DEFINE ***/
//#define	CHECK_EN

/* GPIO CONTROL REGISTERS (EALLOW PROTECTED) */
#define	GPACTRL			(volatile unsigned long *)0x6F80  /* GPIO A Control Register (GPIO0 to 31)				*/
#define	GPAQSEL1		(volatile unsigned long *)0x6F82  /* GPIO A Qualifier Select 1 Register (GPIO0 to 15)	*/
#define	GPAQSEL2		(volatile unsigned long *)0x6F84  /* GPIO A Qualifier Select 2 Register (GPIO16 to 31)	*/
#define	GPAMUX1			(volatile unsigned long *)0x6F86  /* GPIO A MUX 1 Register (GPIO0 to 15)				*/
#define	GPAMUX2			(volatile unsigned long *)0x6F88  /* GPIO A MUX 2 Register (GPIO16 to 31)				*/
#define	GPADIR			(volatile unsigned long *)0x6F8A  /* GPIO A Direction Register (GPIO0 to 31)			*/
#define	GPAPUD			(volatile unsigned long *)0x6F8C  /* GPIO A Pull Up Disable Register (GPIO0 to 31)		*/

#define	GPBCTRL			(volatile unsigned long *)0x6F90  /* GPIO B Control Register (GPIO32 to 44)				*/
#define	GPBQSEL1		(volatile unsigned long *)0x6F92  /* GPIO B Qualifier Select 1 Register (GPIO32 to 44)	*/
#define	GPBMUX1			(volatile unsigned long *)0x6F96  /* GPIO B MUX 1 Register (GPIO32 to 44)				*/
#define	GPBDIR			(volatile unsigned long *)0x6F9A  /* GPIO B Direction Register (GPIO32 to 44)			*/
#define	GPBPUD			(volatile unsigned long *)0x6F9C  /* GPIO B Pull Up Disable Register (GPIO32 to 44)		*/

#define	AIOMUX1			(volatile unsigned long *)0x6FB6  /* Analog, I/O mux 1 register (AIO0 to AIO15)			*/
#define	AIODIR			(volatile unsigned long *)0x6FBA  /* Analog, I/O Direction Register (AIO0 to AIO15)		*/

/* GPIO DATA REGISTERS (NOT EALLOW PROTECTED) */
#define	GPADAT			(volatile unsigned long *)0x6FC0  /* GPIO A Data Register (GPIO0 to 31)					*/
#define	GPASET			(volatile unsigned long *)0x6FC2  /* GPIO A Data Set Register (GPIO0 to 31)				*/
#define	GPACLEAR		(volatile unsigned long *)0x6FC4  /* GPIO A Data Clear Register (GPIO0 to 31)			*/
#define	GPATOGGLE		(volatile unsigned long *)0x6FC6  /* GPIO A Data Toggle Register (GPIO0 to 31)			*/

#define	GPBDAT			(volatile unsigned long *)0x6FC8  /* GPIO B Data Register (GPIO32 to 44)				*/
#define	GPBSET			(volatile unsigned long *)0x6FCA  /* GPIO B Data Set Register (GPIO32 to 44)			*/
#define	GPBCLEAR		(volatile unsigned long *)0x6FCC  /* GPIO B Data Clear Register (GPIO32 to 44)			*/
#define	GPBTOGGLE		(volatile unsigned long *)0x6FCE  /* GPIO B Data Toggle Register (GPIO32 to 44)			*/

#define	AIODAT			(volatile unsigned long *)0x6FD8  /* Analog I/O Data Register (AIO0 to AIO15)			*/
#define	AIOSET			(volatile unsigned long *)0x6FDA  /* Analog I/O Data Set Register (AIO0 to AIO15)		*/
#define	AIOCLEAR		(volatile unsigned long *)0x6FDC  /* Analog I/O Data Clear Register (AIO0 to AIO15)		*/
#define	AIOTOGGLE		(volatile unsigned long *)0x6FDE  /* Analog I/O Data Toggle Register (AIO0 to AIO15)	*/

/* GPIO INTERRUPT AND LOW POWER MODES SELECT REGISTERS (EALLOW PROTECTED) */
#define	GPIOXINT1SEL	(volatile unsigned short *)0x6FE0  /* XINT1 GPIO Input Select Register (GPIO0 to 31)	*/
#define	GPIOXINT2SEL	(volatile unsigned short *)0x6FE1  /* XINT2 GPIO Input Select Register (GPIO0 to 31)	*/
#define	GPIOXINT3SEL	(volatile unsigned short *)0x6FE2  /* XINT3 GPIO Input Select Register (GPIO0 to 31)	*/
#define	GPIOLPMSEL		(volatile unsigned long *)0x6FE8   /* LPM GPIO Select Register (GPIO0 to 31)			*/

/* External Interrupt Control Register */
#define	XINT1CR			(volatile unsigned short *)0x7070	/* XINT1 configuration register */
#define	XINT2CR			(volatile unsigned short *)0x7071	/* XINT2 configuration register */
#define	XINT3CR			(volatile unsigned short *)0x7072	/* XINT3 configuration register */

/*** FUNCTION ***/

/*** WORK ***/

/*******************************************************************************
 *
 */
int gpioInit( void )
{
	return 0;
}


/*******************************************************************************
 *
 */
int gpioConfig( int no, int mode )
{
	int intLockVal;
	unsigned long mask, mask2;
	unsigned long regVal;

	unsigned long mux;
	unsigned long dir;
	unsigned long pullup;
	unsigned long qualifier;

	int	shift;
	int	shift2;
	volatile unsigned long * pDirectionReg;
	volatile unsigned long * pPullUpDisableReg;
	volatile unsigned long * pQualifierSelectReg;
	volatile unsigned long * pMuxReg;

	volatile unsigned short * pXintCrReg;
	unsigned short xintCrVal;
	volatile unsigned short * pGpioXintSelReg;

	switch( no >> 4 ) {
	case 0:	/* no = 0 to 15 */
		shift  = no;
		shift2 = no<<1;

		pDirectionReg       = GPADIR;
		pPullUpDisableReg   = GPAPUD;
		pQualifierSelectReg = GPAQSEL1;
		pMuxReg             = GPAMUX1;
		break;

	case 1:	/* no = 16 to 31 */
		shift  = no;
		shift2 = (no-16)<<1;

		pDirectionReg       = GPADIR;
		pPullUpDisableReg   = GPAPUD;
		pQualifierSelectReg = GPAQSEL2;
		pMuxReg             = GPAMUX2;
		break;

	case 2:	/* no = 32 to 47 */
		shift  = no - 32;
		shift2 = (no-32)<<1;

		pDirectionReg       = GPBDIR;
		pPullUpDisableReg   = GPBPUD;
		pQualifierSelectReg = GPBQSEL1;
		pMuxReg             = GPBMUX1;
		break;

	default :
		return -1;
		/* break; un-reach */
	}

	mask = ~(1UL<<no);
	mask2 = ~(3UL<<shift2);

	switch( mode & (GPIO_MODE_GPIO|GPIO_MODE_1|GPIO_MODE_2|GPIO_MODE_3) ) {
	case GPIO_MODE_GPIO: mux = 0; break;
	case GPIO_MODE_1:    mux = 1; break;
	case GPIO_MODE_2:    mux = 2; break;
	case GPIO_MODE_3:    mux = 3; break;
	default: return -1;
	}

	switch( mode & (GPIO_MODE_IN|GPIO_MODE_OUT) ) {
	case GPIO_MODE_IN:  dir = 0; break;
	case GPIO_MODE_OUT: dir = 1; break;
	default: return -1;
	}

	switch( mode & (GPIO_MODE_PULLUP_DIS|GPIO_MODE_PULLUP) ) {
	case GPIO_MODE_PULLUP_DIS: pullup = 1; break;
	case 0: /* break; */
	case GPIO_MODE_PULLUP:     pullup = 0; break;
	default: return -1;
	}


	switch( mode & (GPIO_MODE_QUALIFIER_0|GPIO_MODE_QUALIFIER_1|GPIO_MODE_QUALIFIER_2|GPIO_MODE_QUALIFIER_3) ) {
	case 0:	/* break; */
	case GPIO_MODE_QUALIFIER_0: qualifier = 0; break;
	case GPIO_MODE_QUALIFIER_1: qualifier = 1; break;
	case GPIO_MODE_QUALIFIER_2: qualifier = 2; break;
	case GPIO_MODE_QUALIFIER_3: qualifier = 3; break;
	default : return -1;
	}

	if( no < 32 ) {
		switch( mode & (GPIO_MODE_XINT1|GPIO_MODE_XINT2|GPIO_MODE_XINT3) ) {
		case GPIO_MODE_XINT1: pXintCrReg = XINT1CR; pGpioXintSelReg = GPIOXINT1SEL; break;
		case GPIO_MODE_XINT2: pXintCrReg = XINT2CR; pGpioXintSelReg = GPIOXINT2SEL; break;
		case GPIO_MODE_XINT3: pXintCrReg = XINT2CR; pGpioXintSelReg = GPIOXINT3SEL; break;
		default:              pXintCrReg = 0;       pGpioXintSelReg = 0;            break;
		}

		switch( mode & (GPIO_MODE_XINTn_FALL|GPIO_MODE_XINTn_RAISE|GPIO_MODE_XINTn_BOTH) ) {
		case GPIO_MODE_XINTn_FALL:  xintCrVal = (2<<2) | 0x01; break;
		case GPIO_MODE_XINTn_RAISE: xintCrVal = (1<<2) | 0x01; break;
		case GPIO_MODE_XINTn_BOTH:  xintCrVal = (3<<2) | 0x01; break;
		default:                    xintCrVal = 0;             break;
		}
	} else {
		pXintCrReg      = 0;
		pGpioXintSelReg = 0;
	}

	dir    <<= shift;
	pullup <<= shift;

	qualifier <<= shift2;
	mux       <<= shift2;

	intLockVal = sysIntLock();
		EALLOW;

			regVal = *pDirectionReg;
			regVal &= mask;
			regVal |= dir;
			*pDirectionReg = regVal;

			regVal = *pPullUpDisableReg;
			regVal &= mask;
			regVal |= pullup;
			*pPullUpDisableReg = regVal;

			regVal = *pQualifierSelectReg;
			regVal &= mask2;
			regVal |= qualifier;
			*pQualifierSelectReg = regVal;

			regVal = *pMuxReg;
			regVal &= mask2;
			regVal |= mux;
			*pMuxReg = regVal;

			if( pXintCrReg ) {
				*pXintCrReg = xintCrVal;
			}

			if( pGpioXintSelReg ) {
				*pGpioXintSelReg = no;
			}

		EDIS;
	sysIntUnlock( intLockVal );

	return 0;
}

/*******************************************************************************
 *
 */
static volatile unsigned long * regTbl[2][4] =
{
	{ GPASET, GPACLEAR, GPATOGGLE, GPADAT, },
	{ GPBSET, GPBCLEAR, GPBTOGGLE, GPBDAT, },
};
int gpioCtrl( int no, int ctrl )
{
	int retVal;
	int intLockVal;
	unsigned long regVal;

	if( no < 32 ) {
		regVal = 1UL<<no;
		no = 0;
	} else {
		regVal = 1UL<<(no-32);
		no = 1;
	}

	retVal = 0;

	if( ctrl != GPIO_CTRL_GET ) {
		intLockVal = sysIntLock();
		*regTbl[no][ctrl] = regVal;
		sysIntUnlock( intLockVal );
	} else {
		if( *regTbl[no][ctrl] & regVal ) {
			retVal = 1;
		}
	}

	return retVal;
}


/*******************************************************************************
 *
 */
int gpioCtrlRegSet( int reg, int _value )
{
	volatile unsigned long * pReg;
	int shift;
	unsigned long mask;
	unsigned long value;
	unsigned long regVal;

	mask = 0x0ff;
	value = (unsigned long)_value;
	value &= mask;
	switch( reg ) {
	case GPIO_CTRLREG_QUALPRD_0_7:			/* GPIO 0-7 */
		pReg = GPACTRL;
		shift = 0;
		break;
	case GPIO_CTRLREG_QUALPRD_8_15:			/* GPIO 8-15 */
		pReg = GPACTRL;
		shift = 8;
		break;
	case GPIO_CTRLREG_QUALPRD_16_23:			/* GPIO 16-23 */
		pReg = GPACTRL;
		shift = 16;
		break;
	case GPIO_CTRLREG_QUALPRD_24_31:			/* GPIO 24-31 */
		pReg = GPACTRL;
		shift = 24;
		break;
	case GPIO_CTRLREG_QUBLPRD_32_39:			/* GPIO 32-39 */
		pReg = GPBCTRL;
		shift = 0;
		break;
	case GPIO_CTRLREG_QUBLPRD_40_44:			/* GPIO 32-44 */
		pReg = GPBCTRL;
		shift = 8;
	default:
		return -1;
	}

	mask  <<= shift;
	value <<= shift;

	mask =~mask;

	EALLOW;

	regVal = *pReg;
	regVal &= mask;
	regVal |= value;
	*pReg = regVal;

	EDIS;

	return 0;
}

/*** end of file : gpioDrv.c ***/
