/*** tiemrDrv.c - TMS320F28035 CPU Timer Driver ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/14 mimu
              First Version
 */

/*** NOTE ***
 * 1) registers
 * TIMER0TIM  0x0C00 1 CPU-Timer 0, Counter Register
 * TIMER0TIMH 0x0C01 1 CPU-Timer 0, Counter Register High
 * TIMER0PRD  0x0C02 1 CPU-Timer 0, Period Register
 * TIMER0PRDH 0x0C03 1 CPU-Timer 0, Period Register High
 * TIMER0TCR  0x0C04 1 CPU-Timer 0, Control Register
 * TIMER0TPR  0x0C06 1 CPU-Timer 0, Prescale Register
 * TIMER0TPRH 0x0C07 1 CPU-Timer 0, Prescale Register High
 *
 * TIMER1TIM  0x0C08 1 CPU-Timer 1, Counter Register
 * TIMER1TIMH 0x0C09 1 CPU-Timer 1, Counter Register High
 * TIMER1PRD  0x0C0A 1 CPU-Timer 1, Period Register
 * TIMER1PRDH 0x0C0B 1 CPU-Timer 1, Period Register High
 * TIMER1TCR  0x0C0C 1 CPU-Timer 1, Control Register
 * TIMER1TPR  0x0C0E 1 CPU-Timer 1, Prescale Register
 * TIMER1TPRH 0x0C0F 1 CPU-Timer 1, Prescale Register High
 *
 * TIMER2TIM  0x0C10 1 CPU-Timer 2, Counter Register
 * TIMER2TIMH 0x0C11 1 CPU-Timer 2, Counter Register High
 * TIMER2PRD  0x0C12 1 CPU-Timer 2, Period Register
 * TIMER2PRDH 0x0C13 1 CPU-Timer 2, Period Register High
 * TIMER2TCR  0x0C14 1 CPU-Timer 2, Control Register
 * TIMER2TPR  0x0C16 1 CPU-Timer 2, Prescale Register
 * TIMER2TPRH 0x0C17 1 CPU-Timer 2, Prescale Register High
 *
 */

/*** INCLUDE ***/
#include "config.h"
#include "sysLib.h"
#include "timerDrv.h"

/*** DEFINE ***/
#ifdef	SUPPORT_CPU_TIMER_0
#  ifdef	SUPPORT_CPU_TIMER_1
#    ifdef	SUPPORT_CPU_TIMER_2
#      define	TIMER_CH_NUM	3
#    else
#      define	TIMER_CH_NUM	2
#    endif	/* SUPPORT_CPU_TIMER_2 */
#  else
#    ifndef	SUPPORT_CPU_TIMER_2
#      define	TIMER_CH_NUM 1
#    endif
#  endif	/* SUPPORT_CPU_TIMER_1 */
#endif	/* SUPPORT_CPU_TIMER_0 */

#ifndef	TIMER_CH_NUM
#error
#endif

#define	BASE_FREQ	60000000UL	/* 60MHz */

typedef struct timer_reg_t {
//	volatile unsigned short time;
//	volatile unsigned short timH;
	volatile unsigned short time32;
//	volatile unsigned short prd;
//	volatile unsigned short prdH;
	volatile unsigned long  prd32;
	volatile unsigned short tcr;
#define	TCR_TIF	(1<<15)		/* CPU-Timer Interrupt Flag. */
#define	TCR_TIE	(1<<14)		/* TIE CPU-Timer Interrupt Enable. */
#define	TCR_TRB	(1<<5)		/* TRB CPU-Timer Reload bit. The TRB bit is always read as zero. Writes of 0 are ignored. */
#define	TCR_TSS	(1<<4)		/* TSS CPU-Timer stop status bit. TSS is a 1-bit flag that stops or starts the CPU-timer. */

	volatile unsigned short tpr;
#define	TPR_PSC(_reg)		((_reg)>>8)		/* 15-8 PSC CPU-Timer Prescale Counter. */
#define	TPR_TDDR(_reg)		((_reg)&0xff)	/* 7-0 TDDR CPU-Timer Divide-Down */

	volatile unsigned short tprH;
} TimerReg;

typedef struct timer_chan_t {
	TimerReg * pReg;
	unsigned long freq;
	int		bConfig;
	int		bRun;
	void (*pCallback)( void * arg );
	void * arg;
//	IntID intID;
	unsigned int vecNo;

	unsigned long base;

} TimerChan;


/*** FUNCTION ***/
static void timerInt( TimerChan * );
static void timer0_int( unsigned int vecNo );
#ifdef	SUPPORT_CPU_TIMER_1
static void timer1_int( unsigned int vecNo );
#endif
#ifdef	SUPPORT_CPU_TIMER_2
static void timer2_int( unsigned int vecNo );
#endif

/*** WORK ***/
static TimerChan timerChanTbl[TIMER_CH_NUM];
static unsigned long baseFreq;

/*******************************************************************************
 *
 */
int timerDrvInit( void )
{
	int ch;
//	IntID intID;
	unsigned int vecNo;
	TimerReg * pRegTop;
	void (*pIsr)( unsigned int );

	baseFreq = BASE_FREQ;

	for( ch=0; ch<TIMER_CH_NUM; ch++ ) {
		switch( ch ) {
#ifdef	SUPPORT_CPU_TIMER_0
//		case 0:	intID = IntID_Timer0; pRegTop = (TimerReg *)0x0c00; break;
		case 0:
			vecNo = 38;
			pRegTop = (TimerReg *)0x0c00;
			pIsr = timer0_int;
			break;
#endif
#ifdef	SUPPORT_CPU_TIMER_1
//		case 1:	intID = IntID_Timer1; pRegTop = (TiemrReg *)0x0c08; break;
		case 1:
			vecNo = 35;
			pRegTop = (TiemrReg *)0x0c08;
			pIsr = timer1_int;
			break;
#endif
#ifdef	SUPPORT_CPU_TIMER_2
//		case 2:	intID = IntID_Timer2; pRegTop = (TiemrReg *)0x0c10; break;
		case 2:
			vecNo = 36;
			pRegTop = (TiemrReg *)0x0c10;
			pIsr = timer2_int;
			break;
#endif
		default:
			return -1;
		}

		timerChanTbl[ch].pReg      = pRegTop;
		timerChanTbl[ch].bConfig   = 0;
		timerChanTbl[ch].bRun      = 0;
//		timerChanTbl[ch].intID     = intID;
		timerChanTbl[ch].vecNo     = vecNo;
		timerChanTbl[ch].pCallback = (void (*)(void *))sysDummyFunc;

		pRegTop->tcr = TCR_TSS;	/* timer stop, disable interrupt */

		if( sysIntConnect( vecNo, pIsr ) ) {
			return -1;
		}

		if( sysIntEnable( vecNo ) ) {
			return -1;
		}
	}


	return 0;
}

/*******************************************************************************
 *
 */
int timerConnect( int ch, void (*pFunc)(void *arg), void * arg )
{
	int intLockVal;

	if( ch >= TIMER_CH_NUM ) {
		return -1;
	}

	intLockVal = sysIntLock();

	timerChanTbl[ch].pCallback = pFunc;
	timerChanTbl[ch].arg       = arg;

	sysIntUnlock( intLockVal );

	return 0;
}

/*******************************************************************************
 *
 */
int timerConfig( int ch, unsigned long freq )
{
	unsigned long  count;
	TimerReg * pReg;

	if( ch >= TIMER_CH_NUM ) {
		return -1;
	}

	if( timerChanTbl[ch].bRun ) {
		return -1;
	}

	count = baseFreq / freq;

	pReg = timerChanTbl[ch].pReg;

	pReg->tcr = TCR_TSS;	/* timer stop, disable interrupt */

	pReg->tpr = 0;
	pReg->tprH = 0;

	pReg->prd32 = count;
//	pReg->prdH = 0;

	pReg->tcr = TCR_TSS	/* timer stop */
	          | TCR_TRB	/* reload timer */
	          | TCR_TIE	/* enable interrupt */
	          ;

	timerChanTbl[ch].base = count;
	return 0;

}

/*******************************************************************************
 *
 */
int timerRun( int ch )
{

	if( ch >= TIMER_CH_NUM ) {
		return -1;
	}

	if( timerChanTbl[ch].bRun ) {
		return -1;
	}

	timerChanTbl[ch].pReg->tcr = TCR_TIE;	/* start timer */
	timerChanTbl[ch].bRun = 1;

	return 0;
}

/*******************************************************************************
 *
 */
int timerStop( int ch )
{

	if( ch >= TIMER_CH_NUM ) {
		return -1;
	}

	timerChanTbl[ch].pReg->tcr = TCR_TIE | TCR_TSS;
	timerChanTbl[ch].bRun = 0;

	return 0;
}

/*******************************************************************************
 *
 */
unsigned long timerBase( int ch )
{
	if( ch >= TIMER_CH_NUM ) {
		return 0;
	}

	return timerChanTbl[ch].base;
}

/*******************************************************************************
 *
 */
unsigned long timerCount( int ch )
{
	unsigned long count;
	TimerReg * pReg;

	if( ch >= TIMER_CH_NUM ) {
		return 0;
	}

	pReg = timerChanTbl[ch].pReg;

	count = timerChanTbl[ch].base;
	count -= pReg->time32;

	return count;
}

/*******************************************************************************
 *
 */
static void timerInt( TimerChan * pChan)
{
//	TimerChan * pChan = (TimerChan *)arg;

	pChan->pCallback( pChan->arg );
}

/*******************************************************************************
 *
 */
static void timer0_int( unsigned int vecNo )
{
	timerInt( &timerChanTbl[0] );
}

#ifdef	SUPPORT_CPU_TIMER_1
static void timer1_int( unsigned int vecNo )
{
	timerInt( &timerChanTbl[1] );
}
#endif

#ifdef	SUPPORT_CPU_TIMER_2
static void timer2_int( unsigned int vecNo )
{
	timerInt( &timerChanTbl[2] );
}
#endif

/*** end of file : timerDrv.c ***/
